/*********************************************************************/
/* @START_COPYRIGHT@                                                 */
/* Statement:     Licensed Materials - Property of IBM               */
/*                                                                   */
/*                5655-A95                                           */
/*                (C) Copyright IBM Corporation. 2006                */
/*                                                                   */
/* Status:        Version 1 Release 0                                */
/* @END_COPYRIGHT@                                                   */
/*                                                                   */
/* Module Name  : CSQ4TRMN                                           */
/*                                                                   */
/* Environment  : MVS BATCH; C/370                                   */
/*                                                                   */
/* Description  : The purpose of this module is to provide a trigger */
/*                monitor which serves a specific trigger            */
/*                initialisation queue and each valid trigger        */
/*                message will be inserted onto the IMS system       */
/*                queue.                                             */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                       Program  Logic                              */
/*                       --------------                              */
/*                                                                   */
/*  main                                                             */
/*  ----                                                             */
/*                                                                   */
/*   call debugLine with welcome message                             */
/*   call GETECB                                                     */
/*   move PCBs into corresponding variables.                         */
/*   call initialise                                                 */
/*   call getParams                                                  */
/*   if getParams unsuccessful                                       */
/*       exit program                                                */
/*   else                                                            */
/*      call buildReport                                             */
/*      if queue manager name is blank                               */
/*         call buildReport                                          */
/*      endif                                                        */
/*      call sendErrorMessage                                        */
/*      call connectToQmgr                                           */
/*      if connected                                                 */
/*          call openintialisationQueue                              */
/*          if opened                                                */
/*              get storage                                          */
/*              if get storage unsuccessful                          */
/*                  call debugLine with error message                */
/*              else                                                 */
/*                  loop until broken                                */
/*                      break out of loop if ECB been posted         */
/*                      call getMessage                              */
/*                      if getMessage unsuccesful                    */
/*                            break out of loop                      */
/*                      endif                                        */
/*                      call processMQTM                             */
/*                      if processMQTM unsuccessful                  */
/*                          break out of loop                        */
/*                      endif                                        */
/*                      call finalise                                */
/*                      if finalise unsuccessful                     */
/*                          break out of loop                        */
/*                      endif                                        */
/*                  endloop                                          */
/*              endif                                                */
/*              call closeQueue for initialiisationQueue             */
/*          endif                                                    */
/*          call disconnectFromQmgr                                  */
/*      endif                                                        */
/*  endif                                                            */
/*  exit program                                                     */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* buildReport                                                       */
/* -----------                                                       */
/*                                                                   */
/*    Builds an error report from the errorcode and other input      */
/*    parameters.                                                    */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* closeQueue                                                        */
/* ----------                                                        */
/*                                                                   */
/*    call MQCLOSE for paramter hqueue                               */
/*    if call successful                                             */
/*        call debugLine with completion and reason codes            */
/*    else                                                           */
/*        call debugLine with error message                          */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* connectToQMgr                                                     */
/* -------------                                                     */
/*                                                                   */
/*    call MQCONN                                                    */
/*    if call unsuccessful                                           */
/*        call debugLine with error message                          */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* debugLine                                                         */
/* ---------                                                         */
/*                                                                   */
/*    Writes debug line to CSQQUT3 if the message debug level if     */
/*    less than or equal to the set debug level                      */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* disconnectFromQMgr                                                */
/* ------------------                                                */
/*                                                                   */
/*    call MQDISC                                                    */
/*    if call unsuccessful                                           */
/*        call debugLine with error message                          */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* endingMessage                                                     */
/* -------------                                                     */
/*                                                                   */
/*    tell IMS that we are ending                                    */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* finalAnalysis                                                     */
/* -------------                                                     */
/*                                                                   */
/*    report the selected input params from what the program has     */
/*    read in.                                                       */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* finalise                                                          */
/* --------                                                          */
/*                                                                   */
/*    finalise matters with IMS                                      */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* getDefaultQMgrName                                                */
/* ------------------                                                */
/*                                                                   */
/*    set open options for queue manager                             */
/*    open queue manager (MQOPEN) for inquire                        */
/*    if open failed                                                 */
/*        display error message                                      */
/*        return from function                                       */
/*    endif                                                          */
/*    call MQINQ for queue manager name                              */
/*    if call successful                                             */
/*        copy the new queue manager name from MQINQ variable        */
/*    endif                                                          */
/*    close the queue manager (MQCLOSE)                              */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* getDLQInformation                                                 */
/* -----------------                                                 */
/*                                                                   */
/*    set open options for queue manager                             */
/*    open queue manager (MQOPEN) for inquire                        */
/*    if open failed                                                 */
/*        call debugLine with error message                          */
/*        return from function                                       */
/*    endif                                                          */
/*    call MQINQ for dead letter queue name                          */
/*    if call successful                                             */
/*        copy the new dead letter queue name from MQINQ variable    */
/*    endif                                                          */
/*    open dead letter queue (MQOPEN) for inquire                    */
/*    if open failed                                                 */
/*        call debugLine with error message                          */
/*        return from function                                       */
/*    endif                                                          */
/*    if call successful                                             */
/*        copy the dead letter queue detials from MQINQ variable     */
/*    endif                                                          */
/*    if dead letter queue details are invalid                       */
/*        call debugLine with error message                          */
/*    endif                                                          */
/*    close the queue manager (MQCLOSE)                              */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* GETECB                                                            */
/* ------                                                            */
/*                                                                   */
/*    an assembler function to return the ECB                        */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* getParams                                                         */
/* ---------                                                         */
/*                                                                   */
/*    open CSQQUT1                                                   */
/*    if open unsuccessful                                           */
/*        return from function                                       */
/*    endif                                                          */
/*    open CSQQUT1                                                   */
/*    while there are lines tom read                                 */
/*        call inputparse for line                                   */
/*        if inputParse unsuccessful                                 */
/*            call showExpectedCSQQUT1                               */
/*            return from function                                   */
/*        endif                                                      */
/*    endwhile                                                       */
/*    call validateInput                                             */
/*    call finalAnalysis                                             */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* getLongMessage                                                    */
/* --------------                                                    */
/*                                                                   */
/*    call buildReport                                               */
/*    call sendErrorMessage                                          */
/*    get extra storage                                              */
/*    call MQGET                                                     */
/*    call debugLine with completion and reason codes                */
/*    if get successful                                              */
/*        call putToDLQ                                              */
/*    else                                                           */
/*        call debugLine with completion and reason codes            */
/*        if connection lost                                         */
/*            call endingMessage                                     */
/*        else                                                       */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* getMessage                                                        */
/* ----------                                                        */
/*                                                                   */
/*    sets up the gmo to perform a MQGET in syncpoint                */
/*    call MQGET                                                     */
/*    call debugLine with completion and reason codes                */
/*    if get successful                                              */
/*        call debugLine with completion and reason codes            */
/*        if message size is unexpected                              */
/*            call debugLine with error message                      */
/*            call buildReport                                       */
/*            call sendErrorMessage                                  */
/*            call putToDLQ                                          */
/*        endif                                                      */
/*    else                                                           */
/*        if connection lost                                         */
/*            call endingMessage                                     */
/*        else if message truncated                                  */
/*            call getLongMessage                                    */
/*        else                                                       */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* initialise                                                        */
/* ----------                                                        */
/*                                                                   */
/*    Initialise the IMS structures and does an INQY                 */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* inputParse                                                        */
/* ----------                                                        */
/*                                                                   */
/*    parses line into key and value                                 */
/*    sets the corresponding key's value in the structure            */
/*    if key is known                                                */
/*        call reportIt for the key and value and true (known).      */
/*    else                                                           */
/*        call reportIt for the key and value and false (unknown).   */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* openDeadQueue                                                     */
/* -------------                                                     */
/*                                                                   */
/*    set open options for queue                                     */
/*    open dead letter queue (MQOPEN) for inquire, output and to     */
/*                        save all context information               */
/*    call debugLine with completion and reason codes                */
/*    if open unsuccessful                                           */
/*        if connection lost                                         */
/*            call endingMessage                                     */
/*        else                                                       */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*    else                                                           */
/*        call debugLine with completion and reason codes            */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* openInitiationQueue                                               */
/* -------------------                                               */
/*                                                                   */
/*    set open options for queue                                     */
/*    open initiation queue (MQOPEN) for input and to save all       */
/*                        context information                        */
/*    call debugLine with completion and reason codes                */
/*    if open unsuccessful                                           */
/*        if connection lost                                         */
/*            call endingMessage                                     */
/*        else                                                       */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*    else                                                           */
/*        call debugLine with completion and reason codes            */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* processMQTM                                                       */
/* -----------                                                       */
/*                                                                   */
/*    if message size is MQTM size                                   */
/*        if strucid incorrect                                       */
/*            call debugLine with strucid don't match                */
/*            call putToDLQ                                          */
/*        else if version incorrect                                  */
/*            call debugLine with wrong version                      */
/*            call putToDLQ                                          */
/*        else if appltype incorrect                                 */
/*            call debugLine with wrong appltype                     */
/*            call putToDLQ                                          */
/*        else                                                       */
/*            call startTrans                                        */
/*        endif                                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* putMsgToDLQ                                                       */
/* -----------                                                       */
/*                                                                   */
/*    set up DLH values                                              */
/*    put DLH onto begining of message                               */
/*    call MQPUT with message                                        */
/*    if put successful                                              */
/*        call debugLine with completion and reason codes            */
/*    else                                                           */
/*        call debugLine with completion and reason codes            */
/*        if connection lost                                         */
/*            call endingMessage                                     */
/*        else                                                       */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*        call buildReport                                           */
/*        call sendErrorMessage                                      */
/*        call buildReport                                           */
/*        call sendErrorMessage                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* putToDLQ                                                          */
/* --------                                                          */
/*                                                                   */
/*    call debugLine with information                                */
/*    if no dead letter queue information exists                     */
/*        call getDLQInformation                                     */
/*    endif                                                          */
/*    if information found                                           */
/*        if dlq not open                                            */
/*             call openDeadQueue                                    */
/*        endif                                                      */
/*        if opened                                                  */
/*            call putMsgToDLQ                                       */
/*        endif                                                      */
/*        call closeQueue with dead letter queue                     */
/*        if error on queue                                          */
/*           call buildReport                                        */
/*           call sendErrorMessage                                   */
/*        endif                                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* reportBadOpen                                                     */
/* --------                                                          */
/*                                                                   */
/*    Report the fact that we couldn't open the CSQQUT1 file         */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* reportIt                                                          */
/* --------                                                          */
/*                                                                   */
/*    Reflect input record onto CSQQUT2 and say whether or not it is */
/*    accepted or not                                                */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* sendErrorMessage                                                  */
/* ----------------                                                  */
/*                                                                   */
/*    change PCB to message PCB                                      */
/*    if change unsuccessful                                         */
/*        call debugLine with information and return code            */
/*        if lterm is MASTER                                         */
/*            change lterm to MASTER                                 */
/*        else                                                       */
/*            send ROLL to IMS                                       */
/*            call debugLine with information and return code        */
/*            return from function                                   */
/*        endif                                                      */
/*    else                                                           */
/*        call debugLine with information and return code            */
/*    endif                                                          */
/*    foreach message segment                                        */
/*        get message segment                                        */
/*        send ISRT and message segment to IMS                       */
/*        if insert unsuccessful                                     */
/*            call debugLine with information and return code        */
/*            send ROLL to IMS                                       */
/*            call debugLine with information and return code        */
/*        else                                                       */
/*            call debugLine with information and return code        */
/*        endif                                                      */
/*    endforeach                                                     */
/*    if using default lterm                                         */
/*        call buildReport                                           */
/*        call sendErrorMessage                                      */
/*    endif                                                          */
/*    send PURG to IMS                                               */
/*    if purge unsuccessful                                          */
/*        call debugLine with information and return code            */
/*    else                                                           */
/*        call debugLine with information and return code            */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* showExpectedCSQQUT1                                               */
/* -------------------                                               */
/*                                                                   */
/*    Shows the expected input/format of the input file CSQQUT1      */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* startTrans                                                        */
/* ----------                                                        */
/*                                                                   */
/*    make MQTMC2 from MQTM                                          */
/*    change PCB to trans PCB                                        */
/*    if change unsuccessful                                         */
/*        call debugLine with information and return code            */
/*        call buildReport                                           */
/*        call sendErrorMessage                                      */
/*    else                                                           */
/*        call debugLine with information and return code            */
/*    endif                                                          */
/*        send ISRT and message to IMS                               */
/*        if insert unsuccessful                                     */
/*            call debugLine with information and return code        */
/*            call buildReport                                       */
/*            call sendErrorMessage                                  */
/*        else                                                       */
/*            send PURG to IMS                                       */
/*            if purge unsuccessful                                  */
/*                call debugLine with information and return code    */
/*                call buildReport                                   */
/*                call sendErrorMessage                              */
/*            else                                                   */
/*                call debugLine with information and return code    */
/*            endif                                                  */
/*        endif                                                      */
/*    endif                                                          */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* UT2Headers                                                        */
/* --------                                                          */
/*                                                                   */
/*    write header line to file CSQQUT2                              */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* validateInput                                                     */
/* -------------                                                     */
/*                                                                   */
/*    Validates the input parameters, signals error if a required    */
/*    paramter is missing and sets the default when required.        */
/*    return from function                                           */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*  writeLine                                                        */
/*  ---------                                                        */
/*                                                                   */
/*   writes the report to a CSQQUT24 if the file is opened           */
/*                                                                   */
/*********************************************************************/
/* Define code CSECT name                                            */
#pragma csect(code,"CSQ4TRMN")
/* Define static CSECT name                                          */
#pragma csect(static,"BCK1WS")
/*Set-up the IMS run options                                         */
#pragma runopts(env(IMS),plist(IMS),trap(OFF),TERMTHDACT(UAIMM))
#pragma variable(stoparea, NORENT)
#pragma linkage(GETECB, OS)

#include <ims.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <cmqc.h>

/*********************************************************************/
/* Set-up #defines                                                   */
/*********************************************************************/
#define WAITINTERVAL 60000
#define BLANK     "                                                "
#define CSQQUT1_BUFFER_LENGTH 82
#define CSQ4TRMN_KEYWORDS_COUNT 4
#define MAX_ERROR_TABLE_SIZE 40
#define REPORT_LINE_LENGTH 120
#define DEBUG_LINE_LENGTH 150

#define DEBUG_LEVEL_DEFAULT 1
#define DEBUG_FILE_ERROR -1
#define DEBUG_LEVEL_NONE 0
#define DEBUG_LEVEL_MINIMUM 1
#define DEBUG_LEVEL_RETURN_CODES 2
#define DEBUG_LEVEL_FULL 3

/*********************************************************************/
/* IMS constants and structures etc.                                 */
/*********************************************************************/
const static char APSB??(5??) = "APSB" ;
const static char CHKP??(5??) = "CHKP" ;
const static char CHNG??(5??) = "CHNG" ;
const static char GU??(5??)   = "GU  " ;
const static char INIT??(5??) = "INIT" ;
const static char ISRT??(5??) = "ISRT" ;
const static char INQY??(5??) = "INQY" ;
const static char PURG??(5??) = "PURG" ;
const static char ROLL??(5??) = "ROLL";
const static char SYNC??(5??) = "SYNC";

typedef struct tagIMSTxt {
  short int imsMsgll;           /* Length field                      */
  short int imsMsgzz;           /* Length field                      */
} IMS_TXT;

typedef struct tagMsgTxt {
  short int msgLen;             /* Length field                      */
  char msgBuff??(120??);        /* Message Text                      */
} MSG_TXT;

typedef struct tagErrorMessageArea {
                                /* Error message Commarea            */
  char msgnum??(4??);           /* Message id to be created          */
  char *insert??(8??);          /* Array of inserts                  */
                                /* insrt(0) normally point           */
                                /* to the user id,                   */
                                /* but for msg 110 and 118           */
                                /* which have only i insert          */
                                /* and are data lines only           */
                                /* (no message number)               */
                                /* pInsrt/sLeng(1) point             */
                                /* to the insert                     */
  IMS_TXT imsMsgTxt;            /* Message for IMS MTO               */
  MSG_TXT msgTxt;
} ERROR_MESSAGE_AREA;

typedef struct tagMsgrb_Lengths {
  int msgrbll;                  /* LL field (length)                 */
  int msgrbzz;                  /* ZZ field (number of               */
                                /* lines in msg if                   */
                                /* multiline                         */
} MSGRB_LENGTHS;

typedef struct tagMsgrBuf {
    MSGRB_LENGTHS msgrhdr;
    char msgrbmsg??(255??);
};

typedef struct tagAIB {
  char id??(8??);               /* Block identifier                  */
  int  len;                     /* Block length                      */
  char sfunc??(8??);            /* function (ENVIRON here)           */
  char rsnm1??(8??);            /* "IOPCB" for us                    */
  char filler1??(16??);
  int  oalen;                   /* IO area length                    */
  int  oause;                   /* Length of data returned           */
  char filler2??(12??);
  int  completion;              /* Completion code                   */
  int  reason;                  /* Reason code                       */
  char filler3??(4??);
  IO_PCB_TYPE *rsel;      /* Returned PCB address              */
  char filler4??(48??);         /* Returned PCB address              */
} AIB;

typedef struct tagAIBIOAREA {
  char IMSid??(8??);             /* IMS ID                           */
  int  IMSrl;
  char cRegType??(8??);
  char aRegType??(8??);
  int  regid;                    /* Region identifier                */
  char programName??(8??);       /* Program name                     */
  char psbName??(8??);           /* PSB name                         */
  char trnName??(8??);           /* Transaction name                 */
  char userid??(8??);            /* User identifier                  */
  char grpName??(8??);           /* Group name                       */
  char giState;                  /* Status group ind                 */
  int  rta;                      /* Recovery token address           */
  int  aparm;                    /* Application parameter address    */
  char sqi??(4??);               /* Shared queue ind                 */
  char useridAS??(8??);          /* Address space user id            */
  char useridI??(1??);           /* Userid indicator                 */
  char filler??(63??);
} AIB_IO_AREA;

typedef struct tagPutApplID {    /* Identifier for DLQH              */
  char paIMSid??(8??);           /* System identifier                */
  char paUserid??(8??);          /* PSB                              */
} PUT_APPL_ID;

typedef struct tagIMSBuffer {
  short int    ll;               /* Length                           */
  short int    zz;               /* Zeros                            */
  MQTMC2 msg;                    /* Trigger Message                  */
} IMS_BUFFER;

/*********************************************************************/
/* Internal data structures                                          */
/*********************************************************************/
typedef struct tagInputParams {
  MQCHAR48 qmgrName;             /* Queue manager name               */
  MQCHAR48 initQueueName;        /* Initiation queue name            */
  char     lterm??(9??);         /* LTERM to remove error message    */
  int      consoleMessages;      /* Send error message to the console*/
  MQCHAR48 consoleMessagesText;  /* Send error message to the console*/
  int      debugLevel;           /* The debug level to use           */
  FILE *   csqqut3Handle;        /* The CSQQUT3 file handle          */
} INPUT_PARAMS;

typedef struct tagQMInfo {
  MQCHAR48 qmgrName;             /* Queue manager name               */
  MQCHAR48 initQueueName;        /* Initiation queue name            */
  MQCHAR48 deadQueueName;        /* Dead letter queue name           */
  MQHCONN hconn;                 /* Queue manager handle             */
  MQHOBJ hinitq;                 /* Initiation queue handle          */
  MQHOBJ hdeadq;                 /* Dead letter queue handle         */
  MQLONG deadMaxSize;            /* Max message size                 */
} QMGR_INFO;

typedef struct tagErrorTable {
  MQCHAR4 msgnum;
  MQCHAR  action;
  MQCHAR128 msgText;
} ERROR_TABLE;

typedef struct tagMessageReport {
  short int ll;                  /* Length of buffer                 */
  short int zz;                  /* Must be 0                        */
  char reportline??(REPORT_LINE_LENGTH??);
  int cc;                        /* Carriage control                 */
} ERROR_REPORT ;

typedef struct tagMessageReportLine {
  short int ll;                  /* Length of buffer                 */
  short int zz;                  /* Must be 0                        */
  char msgBuffer??(REPORT_LINE_LENGTH??);
} REPORT_LINE;

/*********************************************************************/
/* Constants                                                         */
/*********************************************************************/
short const TRUE                  = 1;
short const FALSE                 = 0;

short const INITIATION_QUEUE      = 0;
short const DEAD_LETTER_QUEUE     = 1;
short const QUEUE_MANAGER         = 2;

char*       MODULE_NAME           = "CSQ4TRMN";
char* const VERSION               = "Supportpac Mxxx v1.0";

short const DLQED                 = 1;

short const RETURN_OK             = 0;
short const RETURN_STOP           = 2;
short const RETURN_WARN           = 4;
short const RETURN_FAIL           = 8;

char* const DEFAULT_QUEUE_MANAGER = "SYSTEM";

char* const DEFAULT_LTERM         = "MASTER  ";

/*********************************************************************/
/*                     Function prototypes                           */
/*********************************************************************/
void buildReport(char *errorcode, void *values[]);
short closeQueue(MQHCONN hconn,
                 MQHOBJ hqueue,
                 short  qtype);
short connectToQmgr(QMGR_INFO *info);
void disconnectFromQmgr(QMGR_INFO *info);
void debugLine(int debuglevel, char *fmt, ...);
void endingMessage();
void finalAnalysis(INPUT_PARAMS *params);
short finalise();
MQLONG getDefaultQMgrName(MQHCONN hconn,
                          MQCHAR48 *defQMgrName);
short getDLQInformation(QMGR_INFO *info);
extern void GETECB(volatile int**);
short getParams (INPUT_PARAMS *params);
short getLongMessage(QMGR_INFO *info,
                     MQTM *msgBuffer,
                     MQLONG *dataLength,
                     MQMD *mqmd);
short getMessage(QMGR_INFO *info,
                 MQTM *databuff,
                 MQLONG *dataLength);
void initialise();
short inputParse(char *buf,
                 INPUT_PARAMS *params);
short openDeadQueue(QMGR_INFO *info);
short openInitiationQueue(QMGR_INFO *info);
short processMQTM(QMGR_INFO *info,
                  MQTM     *msgBuffer,
                  MQLONG   msgLength);
short putMsgToDLQ(QMGR_INFO *info,
                  char     *msgBuffer,
                  MQLONG   msgBufferLen,
                  MQLONG   dlqReason);
short putToDLQ(QMGR_INFO *info,
               MQTM *msgBuffer,
               MQLONG msgLength,
               MQLONG dlReason);
void reportBadOpen();
void reportIt(char *buff, int excepted);
short sendErrorMessage();
void showExpectedCSQQUT1();
short startTrans(QMGR_INFO *info,
                 MQTM      *msgBuffer,
                 MQLONG    msgBufferLen);
void UT2Headers();
short validateInput(INPUT_PARAMS *params);
void writeLine(FILE *csqqut2);

/*********************************************************************/
/* Declare and initialise message table.                             */
/*                                                                   */
/* Continue char \                                                   */
/*********************************************************************/
ERROR_TABLE msgtable??(MAX_ERROR_TABLE_SIZE??) =
  {
    { "Q100",
      'I',
      "%s %s Processing queue manager %s"
    },
    { "Q101",
      'E',
      "%s %s Cannot open the initiation queue, MQCC=%d MQRC=%d"
    },
    { "Q102",
      'E',
      "%s An IMS %s call returned %X"
    },
    { "Q103",
      'E',
      "%s %s CSQ4TRMN read a trigger message with an " \
        "incorrect MQTM-StrucId of %s"
    },
    { "Q104",
      'E',
      "%s %s CSQ4TRMN does not support version %d"
    },
    { "Q105",
      'E',
      "%s %s CSQ4TRMN cannot start a process type of %d"
    },
    { "Q106",
      'E',
      "%s %s MQGET error, MQCC=%d MQRC=%d. CSQ4TRMN will end"
    },
    { "Q107",
      'E',
      "%s %s Cannot connect to the queue manager, MQCC=%d MQRC=%d"
    },
    { "Q108",
      'E',
      "%s %s LTERM %s not available. Switched to MASTER"
    },
    { "Q109",
      'E',
      "%s %s MQCLOSE error, MQCC=%d MQRC=%d"
    },
    { "Q110",
      'I',
      "Queue name = %s"
    },
    { "Q111",
      'E',
      "%s %s CSQ4TRMN read a trigger message with an " \
        "incorrect length of %d"
    },
    { "Q112",
      'E',
      "%s %s MQOPEN error, MQCC=%d MQRC=%d"
    },
    { "Q113",
      'I',
      "%s %s This message cannot be processed"
    },
    { "Q114",
      'E',
      "%s %s MQINQ error, MQCC=%d MQRC=%d"
    },
    { "Q115",
      'I',
      "%s %s Ending following termination of queue manager connection"
    },
    { "Q116",
      'E',
      "%s %s Cannot open the queue manager, MQCC=%d MQRC=%d"
    },
    { "Q117",
      'E',
      "%s %s Cannot query the queue manager. MQCC=%d MQRC=%d"
    },
    { "Q118",
      'I',
      "MsgID=%X"
     },
    { "Q119",
      'E',
      "%s %s Error %d from STORAGE OBTAIN"
    },
    { "Q120",
      'E',
      "%s %s MQPUT error, MQCC=%d MQRC=%d"
    },
    { "Q121",
      'E',
      "%s %s Dead-letter queue is not defined for the queue manager"
    },
    { "Q122",
      'E',
      "%s %s Cannot close the queue manager, MQCC=%d MQRC=%d"
    },
    { "Q123",
      'E',
      "%s %s The dead-letter queue type is not QLOCAL"
    },
    { "Q124",
      'E',
      "%s %s The dead-letter queue usage is not NORMAL"
    },
    { "Q125",
      'E',
      "%s %s No initiation queue identified"
    },
    { "Q126",
      'E',
      "%s %s %s An IMS %s call returned %s"
    },
    { "Q150",
      'I',
      "%s IBM WebSphere MQ for z/OS %s"
    },
    { "Q151",
      'I',
      "%s Trigger Monitor Input Report - %s %s "
    },
    { "Q152",
      'I',
      "%s Unable to OPEN CSQQUT1 data set"
    },
    { "Q153",
      'I',
      "%s First token is not a valid keyword"
    },
    { "Q159",
      'I',
      "%s Trigger monitor options:"
    },
  };

/*********************************************************************/
/* Globel varibles                                                   */
/*********************************************************************/
ERROR_REPORT report;
ERROR_MESSAGE_AREA messagereport;
FILE *csqqut2;
INPUT_PARAMS params;
AIB aib;
AIB_IO_AREA aibioarea;
PUT_APPL_ID putApplid;
IMS_BUFFER ims_buffer;

long parmCount;

char psbName??(8??);

volatile int *ecb;

IO_PCB_TYPE *ioPCB;                               /* IO PCB          */
IO_PCB_TYPE *transPCB;                            /* Transaction PCB */
IO_PCB_TYPE *msgPCB;                              /* Message PCB     */

int main() {
  MQHCONN hconn;
  MQHOBJ  htrigq;
  ERROR_MESSAGE_AREA ermsg;
  QMGR_INFO info;
  MQLONG msgLength;
  MQTM *msgBuffer;
  short rc;
  short temprc;
  int returnvalue = RETURN_OK;
  void *values[4];
  int checkecb = 0;

  params.debugLevel = DEBUG_LEVEL_FULL;
  debugLine(DEBUG_LEVEL_FULL,
             "Started IMS Trigger Monitor (%s) Version %s",
             MODULE_NAME,
             VERSION);
  debugLine(DEBUG_LEVEL_FULL, "Setting up ECB");
  GETECB(&ecb);

  memset(params.qmgrName, '\0', MQ_Q_MGR_NAME_LENGTH);
  memset(params.initQueueName, '\0', MQ_Q_NAME_LENGTH);
  memset(params.lterm, ' ', 8);
  params.lterm??(8??) = '\0';
  params.consoleMessages = 0;
  ioPCB = (IO_PCB_TYPE *) (__pcblist??(0??));
  transPCB = (IO_PCB_TYPE *) (__pcblist??(1??));
  msgPCB = (IO_PCB_TYPE *) (__pcblist??(2??));

  report.ll = 0;
  memset(report.reportline, '\0', REPORT_LINE_LENGTH);
  report.zz = 0;

  initialise();

  rc = getParams(&params);
  if (rc >= RETURN_STOP) {
    return rc;
  }

  /********************************************************************/
  /*                                                                  */
  /* Issue starting message(s)                                        */
  /*                                                                  */
  /********************************************************************/

  values??(0??) = &putApplid.paUserid;
  values??(1??) = &aibioarea.regid;
  values??(2??) = &params.qmgrName;
  values??(3??) = NULL;
  if (rc == 0) {
    debugLine(DEBUG_LEVEL_FULL, "About to call buildReport() for Q100");
    buildReport("Q100",
                values);
    if (params.qmgrName??(0??) == ' ') {
      values??(2??) = DEFAULT_QUEUE_MANAGER;
      buildReport("Q100",
                  values);
    }
    debugLine(DEBUG_LEVEL_FULL, "About to send Q100");
    rc = sendErrorMessage();
    values??(0??) = &params.initQueueName;
    values??(1??) = NULL;
    values??(2??) = NULL;
    values??(3??) = NULL;
    buildReport("Q110",
                values);
    temprc = sendErrorMessage();
    if (rc < temprc) {
      rc = temprc;
    }
    /******************************************************************/
    /*                                                                */
    /* Connect to Queue Manager                                       */
    /*                                                                */
    /******************************************************************/
    memcpy(info.qmgrName,
           params.qmgrName,
           MQ_Q_MGR_NAME_LENGTH);
    memcpy(info.initQueueName,
           params.initQueueName,
           MQ_Q_NAME_LENGTH);
    info.deadMaxSize = -1;
    rc = connectToQmgr(&info);
    if (rc == RETURN_OK) {

      /****************************************************************/
      /*                                                              */
      /* Open the Initiation queue                                    */
      /*                                                              */
      /****************************************************************/
      rc = openInitiationQueue(&info);
      if (rc == RETURN_OK) {

        /**************************************************************/
        /*                                                            */
        /* GetWait Loop                                               */
        /*                                                            */
        /* In this loop do a get wait on the initiation queue until   */
        /* the reason code (qmgr stopping/quiescing and so on)        */
        /* indicates that we should end.                              */
        /*                                                            */
        /**************************************************************/
        msgBuffer = (MQTM *)malloc(sizeof(MQTM));
        while (1) {
          /* Malloc the storage required. */
          if (msgBuffer == NULL) {
            debugLine(DEBUG_LEVEL_MINIMUM,
                      "ERROR: Unable to allocate storage for buffer.");
            break;
          }
          else {
            /* Get wait a message */
            checkecb = *ecb & 0x40000000;
            if (checkecb > 0) {
              free(msgBuffer);
              rc = RETURN_OK;
              break;
            }
            rc = getMessage(&info, msgBuffer, &msgLength);
            if ((rc == DLQED) || (rc == RETURN_WARN)){
              continue;
            }
            else if (rc != RETURN_OK) {
              break;
            }
            temprc = processMQTM(&info,
                                 msgBuffer,
                                 msgLength);
            if (temprc == RETURN_STOP) {
              rc = RETURN_STOP;
              break;
            }
            else if (temprc > rc) {
              rc = temprc;
            }
            temprc = finalise();
            if (temprc >= RETURN_STOP) {
              rc = RETURN_STOP;
              break;
            }
            else if (temprc > rc) {
              rc = temprc;
            }
          }
        } /* end while (1) */
        free(msgBuffer);
      }  /* end if (rc == 0) */
      closeQueue(info.hconn,
                 info.hinitq,
                 INITIATION_QUEUE);
    } /* end if (rc == 0) */
    disconnectFromQmgr(&info);
  }

  return rc;
}

/*********************************************************************/
/*                                                                   */
/* Function Name:  buildReport                                       */
/*                                                                   */
/* Description: Prepares the report structure for SYSPRINT.          */
/*              CSQQUT1.                                             */
/*                                                                   */
/* Notes: the inserts are void pointers and the text in the message  */
/*        tells us was type each one should be, they are either %s,  */
/*        %d or %X. We need to copy the characters accross until we  */
/*        find %, then act appropriately.                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  errorcode         - ptr to the error code.     */
/*                    values            - an array of 4 void ptrs    */
/*                                        that contain error data    */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void buildReport(char *errorcode, void *values[]) {
  char buff??(REPORT_LINE_LENGTH??);
  char tempbuff??(REPORT_LINE_LENGTH??);
  short offset;
  short charloop;
  short loop;
  short length;
  short cvalues;
  char *msgptr;
  void *data;

  loop = 0;
  offset = 0;
  memset(buff, ' ', REPORT_LINE_LENGTH);
  if (errorcode??(2??) == '5') {
    debugLine(DEBUG_LEVEL_FULL,
              "Building report for code %s\n",
              errorcode);
  }
  else {
    debugLine(DEBUG_LEVEL_FULL,
              "Building error report for error code %s\n",
              errorcode);
  }
  while ((strncmp(errorcode,
                  msgtable??(loop??).msgnum,
                  sizeof(MQCHAR4)) !=0) &&
         (loop < MAX_ERROR_TABLE_SIZE)) {
    loop++;
  }
  if (loop < MAX_ERROR_TABLE_SIZE) {
    strncpy(&buff??(offset??), MODULE_NAME, 4);
    offset = 4;
    sprintf(&buff??(offset??), "%s", errorcode);
    offset += 4;
    if (msgtable??(loop??).action == 'E') {
      sprintf(&buff??(offset??), "E ");
    }
    else {
      sprintf(&buff??(offset??), "I ");
    }       
    offset += 2;
    msgptr = msgtable??(loop??).msgText;
    for (charloop = 0, cvalues = 0;
         charloop <= strlen(msgtable??(loop??).msgText);
         charloop++)
    {
      if (msgptr??(charloop??) == '%')
      {
        memset(tempbuff, '\0', REPORT_LINE_LENGTH);
        if (msgptr??(charloop + 1??) == 'd')
        {
          sprintf(tempbuff, "%d", *(int *)(values[cvalues]));
        }
        else if (msgptr??(charloop + 1??) == 's')
        {
          sprintf(tempbuff, "%s", values[cvalues]);
        }
        else if (msgptr??(charloop + 1??) == 'X') {
          sprintf(tempbuff, "%X", (int *)values[cvalues]);
        }
        length = strlen(tempbuff);
        memcpy(&buff??(offset??), tempbuff, length);
        offset += sizeof(char) * length;
        charloop++;
        cvalues++;
      }
      else {
        memcpy(&buff??(offset??), &msgptr??(charloop??), sizeof(char));
        offset += sizeof(char);
      }
    }
    strncpy(report.reportline, buff, REPORT_LINE_LENGTH);
    report.ll = strlen(buff);
    report.zz = 0;
    debugLine(DEBUG_LEVEL_FULL, report.reportline);
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "ERROR: Unknown message type %s.",
              errorcode);
  }
}  
/*********************************************************************/
/* End of function buildReport                                       */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  closeQueue                                        */
/*                                                                   */
/* Description: MQCLOSE the queue                                    */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  hconn            - The queue manager handle.   */
/*                    hqueue           - The queue handle.           */
/*                    qtype            - Type of queue (see above).  */
/*                                                                  */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if the close was successful */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short closeQueue(MQHCONN hconn,
                 MQHOBJ hqueue,
                 short  qtype) {
  MQLONG    mqcc = 0;                  /* Completion code            */
  MQLONG    mqrc = 0;                  /* Reason code                */
  short     rc   = RETURN_OK;
  void      *values[4];

  debugLine(DEBUG_LEVEL_FULL, "Closing Queue of type %d", qtype);
  MQCLOSE(hconn,                      /* Queue Manager handle        */
          &hqueue,                    /* Queue handle                */
          MQCO_NONE,                  /* Open options                */
          &mqcc,                      /* Completion code             */
          &mqrc);                     /* Reason code                 */

  if (mqcc != MQCC_OK) {
    /* MQCLOSE was not successful. */
    if (qtype == INITIATION_QUEUE) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "ERROR closing initiation queue: CC - %ld, RC - %ld",
                mqcc,
                mqrc);
    } /* (qtype == INITIATION_QUEUE) */
    else if (qtype == DEAD_LETTER_QUEUE) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "ERROR closing dead letter queue: CC - %ld, RC - %ld",
                mqcc,
                mqrc);
     if ((mqrc == MQRC_Q_MGR_STOPPING) ||
          (mqrc == MQRC_Q_MGR_QUIESCING) ||
          (mqrc == MQRC_CONNECTION_BROKEN) ||
          (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
        endingMessage();
        rc = RETURN_STOP;
      }
      else {
        values??(0??) = &putApplid.paUserid;
        values??(1??) = &aibioarea.regid;
        values??(2??) = &mqcc;
        values??(3??) = &mqrc;
        buildReport("Q109",
                    values);
        sendErrorMessage();
        rc = RETURN_FAIL;
      }
    } /* (qtype == DEAD_LETTER_QUEUE) */
    else if (qtype == QUEUE_MANAGER) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                 "ERROR closing queue manager: CC - %ld, RC - %ld",
                 mqcc,
                 mqrc);
      if ((mqrc == MQRC_Q_MGR_STOPPING) ||
          (mqrc == MQRC_Q_MGR_QUIESCING) ||
          (mqrc == MQRC_CONNECTION_BROKEN) ||
          (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
        endingMessage();
        rc = RETURN_STOP;
      }
      else {
        values??(0??) = &putApplid.paUserid;
        values??(1??) = &aibioarea.regid;
        values??(2??) = &mqcc;
        values??(3??) = &mqrc;
        buildReport("Q122",
                    values);
        sendErrorMessage();
        rc = RETURN_FAIL;
      }
    } /* (qtype == QUEUE_MANAGER) */
  } /* end if (mqcc != MQCC_OK) */
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
               "Closing queue (type %d): CC - %ld, RC - %ld",
               qtype,
               mqcc,
               mqrc);
  }

  return rc;
}
/*********************************************************************/
/* End of function closeQueue                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: connectToQmgr                                      */
/*                                                                   */
/* Description: MQCONN the queue manager.                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none.                                          */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  info         - Ptr to the MQ infomation        */
/*                                                                   */
/* Returns:           RETURN_OK    - If the queue manager was        */
/*                                   connected to.                   */
/*                    RETURN_STOP  - Otherwise.                      */
/*                                                                   */
/*********************************************************************/
short connectToQmgr(QMGR_INFO *info) {
  MQLONG    mqcc = 0;                 /* Completion code             */
  MQLONG    mqrc = 0;                 /* Reason code                 */
  short     rc   = RETURN_OK;         /* Return code                 */
  void      *values[4];               /* Build report arguements     */

  debugLine(DEBUG_LEVEL_FULL,
            "Connecting to queue manager %s",
            info->qmgrName);
  MQCONN(&info->qmgrName??(0??),      /* Queue Manager name          */
         &info->hconn,                /* Object descriptor           */
         &mqcc,                       /* Completion code             */
         &mqrc);                      /* Reason code                 */

  if (mqcc != MQCC_OK) {
    /* MQCONN was NOT successful */
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR when connecting to queue manager " \
              "(mqcc %d mqrc %d)",
              mqcc,
              mqrc);
    /*                                                               */
    /* If no queue manager has been input get the default queue      */
    /* manager                                                       */
    /*                                                               */
    if (strncmp(&info->qmgrName??(0??),
                BLANK,
                MQ_Q_MGR_NAME_LENGTH) == 0) {
      rc = getDefaultQMgrName(info->hconn, &info->qmgrName);
      if (rc != MQCC_OK) {
        values??(0??) = &putApplid.paUserid;
        values??(1??) = &aibioarea.regid;
        values??(2??) = &mqcc;
        values??(3??) = &mqrc;
        buildReport("Q107",
                    values);
        sendErrorMessage();
        rc = RETURN_STOP;
      }
    }
    else {
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q107",
                  values);
      sendErrorMessage();
      rc = RETURN_STOP;
    }
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Connected to queue manager with mqcc %d mqrc %d",
              mqcc,
              mqrc);
  }

  return rc;
}
/*********************************************************************/
/* End of function connectToQmgr                                     */
/*********************************************************************/
  
/*********************************************************************/
/*                                                                   */
/* Function Name: debugLine                                          */
/*                                                                   */
/* Description: Writes debug line to CSQQUT3 if the message debug    */
/*              level if less than or equal to the set debug level.  */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  debuglevel   - the debug level related to      */
/*                                   message.                        */
/*                    *fmt         - character arrey of the message  */
/*                                   and format (similar to printf)  */
/*                                   %d, %ld, %x, %s and %c allowed. */
/*                    ...          - zero or more parameters.        */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  none.                                          */
/*                                                                   */
/* Returns:           none.                                          */
/*                                                                   */
/*********************************************************************/
void debugLine(int debuglevel, char *fmt, ...)
{
  char buff??(DEBUG_LINE_LENGTH??);
  time_t timenow;
  char tempbuff??(DEBUG_LINE_LENGTH??);
  char *bufptr;
  char *msgptr;
  int loop;
  int length;
  char *pos;
  va_list arg_ptr;
  char timebuff??(10??);
  struct tm *ptr;
  short offset;

  if (debuglevel <= params.debugLevel) {
    memset(buff, ' ', DEBUG_LINE_LENGTH);
    time(&timenow);
    ptr = localtime(&timenow);
    strftime(timebuff, 10, "%X", ptr);
    bufptr = fmt;
    va_start(arg_ptr, fmt);
    bufptr = fmt;
    offset = 0;
    length = strlen(timebuff);
    memcpy(&buff??(offset??), timebuff, length);
    offset += length + 1;
    for (loop = 0; loop <= strlen(fmt); loop++)
    {
      if (fmt??(loop??) == '%')
      {
        memset(tempbuff, '\0', DEBUG_LINE_LENGTH);
        if (fmt??(loop + 1??) == 'd')
        {
          sprintf(tempbuff, "%d", va_arg(arg_ptr, int));
        }
        else if (fmt??(loop + 1??) == 'x')
        {
          sprintf(tempbuff, "%x", va_arg(arg_ptr, int));
        }
        else if ((fmt??(loop + 1??) == 'l') &&
                 (fmt??(loop + 2??) == 'd'))
        {
          sprintf(tempbuff, "%ld", va_arg(arg_ptr, long));
          loop++;
        }
        else if (fmt??(loop + 1??) == 's')
        {
          sprintf(tempbuff, "%s", va_arg(arg_ptr, char *));
        }
        else if (msgptr??(loop + 1??) == 'c') {
          sprintf(tempbuff, "%c", va_arg(arg_ptr, char));
        }
        length = strlen(tempbuff);
        memcpy(&buff??(offset??), tempbuff, length);
        offset += sizeof(char) * length;
        loop++;
      }
      else {
        memcpy(&buff??(offset??), &fmt??(loop??), sizeof(char));
        offset += sizeof(char);
      }
    }
    fprintf(params.csqqut3Handle, "%s\n", buff);
    va_end(arg_ptr);
  }
}
/*********************************************************************/
/* End of function debugLine                                         */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: disconnectFromQmgr                                 */
/*                                                                   */
/* Description: MQDISC the queue manager.                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info         - Ptr to the MQ infomation        */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  none.                                          */
/*                                                                   */
/* Returns:           none.                                          */
/*                                                                   */
/*********************************************************************/
void disconnectFromQmgr(QMGR_INFO *info) {
  MQLONG    mqcc = 0;                  /* Completion code            */
  MQLONG    mqrc = 0;                  /* Reason code                */

  MQDISC(&info->hconn,                 /* Object descriptor           */
         &mqcc,                        /* Completion code             */
         &mqrc);                       /* Reason code                 */
  if (mqcc == MQCC_OK) {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Disconnecting from queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
  }
  else {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "Disconnecting from  queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
  }
}
/*********************************************************************/
/* End of function disconnectFromQmgr                                */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: EndingMessage                                      */
/*                                                                   */
/* Description: Report the fact that we are ending                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void endingMessage() {
  void *values[4] = {NULL, NULL, NULL, NULL};

  buildReport("Q115",
              values);
  sendErrorMessage();
}
/*********************************************************************/
/* End of function EndingMessage                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  finalAnalysis                                     */
/*                                                                   */
/* Description: Report the selected input params from what the       */
/*              program has read in.                                 */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  params       - ptr to a structure to hold the  */
/*                                   input parameters.               */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/

void finalAnalysis(INPUT_PARAMS *params) {
  void *values[4];

  values??(0??) = MODULE_NAME;
  values??(1??) = VERSION;
  values??(2??) = NULL;
  values??(3??) = NULL;

  buildReport("Q159",
              values);
  writeLine(csqqut2);

  memset(report.reportline, '\0', REPORT_LINE_LENGTH);
  sprintf(report.reportline,
          "QMGRNAME        = %s",
          params->qmgrName);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, report.reportline);
  memset(report.reportline, '\0', REPORT_LINE_LENGTH);
  sprintf(report.reportline,
          "INITQUEUENAME   = %s",
          params->initQueueName);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, report.reportline);
  memset(report.reportline, '\0', REPORT_LINE_LENGTH);
  sprintf(report.reportline,
          "LTERM           = %s",
          params->lterm);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, report.reportline);
  memset(report.reportline, '\0', REPORT_LINE_LENGTH);
  sprintf(report.reportline,
          "CONSOLEMESSAGES = %s",
          params->consoleMessagesText);
  debugLine(DEBUG_LEVEL_FULL, report.reportline);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, "Leaving finalAnalysis()");
}
/*********************************************************************/
/* End of function finalAnalysis                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: finalise                                           */
/*                                                                   */
/* Description: Finalises matters with IMS.                          */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
short finalise() {
  int ctdliRC;
  int rc = RETURN_OK;
  void *values[4];

  parmCount = 2;
  ctdliRC = CTDLI(&parmCount, SYNC, ioPCB);
  debugLine(DEBUG_LEVEL_RETURN_CODES,
            "%x returned on SYNC command using CTDLI()",
            ctdliRC);
  switch (ctdliRC) {
  case 0    : break;                              /* Successful call */
  case 0x00e7c4 : /* 'XD' */
                  /* CHE FREEZE issued */
              rc = RETURN_FAIL;
              break;
  default:
      if (params.debugLevel < DEBUG_LEVEL_RETURN_CODES) {
        debugLine(DEBUG_LEVEL_MINIMUM,
                  "%x returned on SYNC command using CTDLI()",
                  ctdliRC);
      }
      values??(0??) = &putApplid.paIMSid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &SYNC;
      values??(3??) = &ctdliRC;
      buildReport("Q126",
                  values);
      sendErrorMessage();
      rc = RETURN_FAIL;
  }

  return rc;
}
/*********************************************************************/
/* End of function finalise                                          */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: getDefaultQMgrName                                 */
/*                                                                   */
/* Description: Opens the queue manager and inquires on the default  */
/*              queue manager name.                                  */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  hconn       - the connection handle.           */
/*                                                                   */
/* Output Parameters: defQMgrName - the default queue manager name.  */
/*                                                                   */
/* InOut Parameters:  none.                                          */
/*                                                                   */
/* Returns:           the MQRC from the last MQI call that was made. */
/*                                                                   */
/*********************************************************************/
MQLONG getDefaultQMgrName(MQHCONN hconn,
                          MQCHAR48 *defQMgrName) {
  void *values[4];
  MQHOBJ   hobj;
  MQLONG   openopts;
  MQOD     objdesc = { MQOD_DEFAULT };
  MQLONG   selectors??(1??);
  MQLONG   intattrs??(1??);
  MQCHAR48 charattrs;
  MQLONG   selectorcount  = 1;
  MQLONG   intattrcount   = 0;
  MQLONG   charattrlength = MQ_Q_MGR_NAME_LENGTH;
  MQLONG mqcc = 0;
  MQLONG mqrc = MQRC_NONE;

  debugLine(DEBUG_LEVEL_FULL,
            "Going to connect to default queue manager");
  /*                                                                */
  /*      Set the open options to inquire on a queue manager        */
  /*                                                                */
  objdesc.ObjectType = MQOT_Q_MGR;
  memset(objdesc.ObjectName, ' ', MQ_Q_MGR_NAME_LENGTH);

  openopts = MQOO_INQUIRE;

  MQOPEN(hconn,
         &objdesc,
         openopts,
         &hobj,
         &mqcc,
         &mqrc);

  /*                                                                */
  /*    If the open was unsuccessful then return the reason code    */
  /*                                                                */
  if (mqcc != MQCC_OK) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR when opening queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
    values??(0??) = &putApplid.paUserid;
    values??(1??) = &aibioarea.regid;
    values??(2??) = &mqcc;
    values??(3??) = &mqrc;
    buildReport("Q107",
                values);
    sendErrorMessage();
    return RETURN_STOP;
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Opening queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
  }
  /*                                                                */
  /*       Set the inquire options to inquire on the queue          */
  /*       manager name and call MQINQ                              */
  /*                                                                */
  selectors??(0??) = MQCA_Q_MGR_NAME;

  MQINQ(hconn,
        hobj,
        selectorcount,
        selectors,
        intattrcount,
        intattrs,
        charattrlength,
        charattrs,
        &mqcc,
        &mqrc);

  /*                                                                */
  /* If the inquire was successful then copy the default            */
  /* queue manager name to variable passed to function              */
  /* else report error and return reason code.                      */
  /*                                                                */
  if (mqcc == MQCC_OK) {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Inquiring queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
    strncpy(defQMgrName??(0??), charattrs, MQ_Q_MGR_NAME_LENGTH);
  }
  else {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "Opening queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
    values??(0??) = &putApplid.paUserid;
    values??(1??) = &aibioarea.regid;
    values??(2??) = &mqcc;
    values??(3??) = &mqrc;
    buildReport("Q107",
                values);
    sendErrorMessage();
    return RETURN_STOP;
  }

  /*                                                                 */
  /*               Close the queue manager                           */
  /*                                                                 */
  MQCLOSE(hconn,
          &hobj,
          MQCO_NONE,
          &mqcc,
          &mqrc);
  if (mqcc == MQCC_OK) {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Closing queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
    strncpy(defQMgrName??(0??), charattrs, MQ_Q_MGR_NAME_LENGTH);
  }
  else {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "Closing queue manager (mqcc %d mqrc %d)",
              mqcc,
              mqrc);
  }

  return mqrc;
}
/*********************************************************************/
/* End of function getDefaultQmgrName                                */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  getDLQInformationOpen                             */
/*                                                                   */
/* Description: get the dead letter queue information.               */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  msgLength    - message length.                 */
/*                    msgBuffer    - Ptr to the message.             */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short getDLQInformation(QMGR_INFO *info) {

  MQCHAR csCharAttrs??(MQ_Q_NAME_LENGTH??);
  MQLONG attrsLength = MQ_Q_NAME_LENGTH;
  MQLONG selectors??(2??);                 /* MQINQ selectors        */
  MQLONG alIntAttrs??(2??);                /* MQINQ selectors        */
  MQLONG csCharLength;
  MQLONG mqcc = 0;                         /* Completion code        */
  MQLONG mqrc = 0;                         /* Reason code            */
  MQOD objDesc = { MQOD_DEFAULT };
  MQHOBJ hqmgr;
  MQLONG openopts;
  short rc = RETURN_OK;
  short len = 0;
  void *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Gathering Dead Letter Queue Information");
  /*                                                                 */
  /* Set the object descriptor fields to open the queue manager      */
  /* attributes object.                                              */
  /*                                                                 */
  objDesc.ObjectType = MQOT_Q_MGR;
  memset(objDesc.ObjectName, ' ', MQ_Q_MGR_NAME_LENGTH);
  memset(csCharAttrs, ' ', MQ_Q_MGR_NAME_LENGTH);

  /*                                                                 */
  /* Set the open options to inquire on a queue manager. The         */
  /* open call should fail if the queue manager is quiescing.        */
  /*                                                                 */

  openopts = MQOO_INQUIRE +
             MQOO_FAIL_IF_QUIESCING;
  MQOPEN(info->hconn,
         &objDesc,
         openopts,
         &hqmgr,
         &mqcc,
         &mqrc);
  if (mqcc != MQCC_OK) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "Error opening queue manager for inquire " \
              "(mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    if ((mqrc == MQRC_Q_MGR_STOPPING) ||
        (mqrc == MQRC_Q_MGR_QUIESCING) ||
        (mqrc == MQRC_CONNECTION_BROKEN) ||
        (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
      endingMessage();
      rc = RETURN_STOP;
    }
    else {
      values??(0??) = &putApplid.paIMSid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q116",
                  values);
      sendErrorMessage();
      rc = RETURN_FAIL;
    } /* mqrc on MQOPEN on Qmgr */
  } /* end if mqcc != MQCC_OK on MQOPEN on Qmgr */
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Opening queue manager for inquire (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    selectors??(0??) = MQCA_DEAD_LETTER_Q_NAME;

    MQINQ(info->hconn,              /* Queue Manager Handle          */
          hqmgr,                    /* Queue Managr enquiry handle.  */
          1,                        /* Number of selectore           */
          selectors,                /* Selectors                     */
          0,                        /* Number of returned Intergers  */
          NULL,                     /* Returned Integers             */
          attrsLength,              /* Length of returned chars      */
          csCharAttrs,              /* Returned chars                */
          &mqcc,                    /* Completion code               */
          &mqrc);                   /* Reason code                   */

    if (mqcc != MQCC_OK) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "Error Inquiring queue manager for dead letter " \
                "queue name (mqcc %ld mqrc %ld)",
                mqcc,
                mqrc);
      if ((mqrc == MQRC_Q_MGR_STOPPING) ||
          (mqrc == MQRC_Q_MGR_QUIESCING) ||
          (mqrc == MQRC_CONNECTION_BROKEN) ||
          (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
        endingMessage();
        rc = RETURN_STOP;
      }
      else {
        values??(0??) = &putApplid.paUserid;
        values??(1??) = &aibioarea.regid;
        values??(2??) = &mqcc;
        values??(3??) = &mqrc;
        buildReport("Q117",
                    values);
        sendErrorMessage();
        rc = RETURN_FAIL;
      } /* mqrc on MQINQ QMgr */
    } /* end if mqcc != MQCC_OK on MQINQ QMgr */
    else {
      /* MQINQ completed successfully */
      debugLine(DEBUG_LEVEL_RETURN_CODES,
                "Inquiring queue manager for dead letter queue name " \
                "(mqcc %ld mqrc %ld)",
                mqcc,
                mqrc);
      /* Get the dead letter queue */
      len = strlen(csCharAttrs);
      memcpy(info->deadQueueName, csCharAttrs, len);
      if (strcmp(info->deadQueueName, " ") == 0) {
        debugLine(DEBUG_LEVEL_MINIMUM,
                   "Dead-letter queue is not defined for the queue " \
                   "manager");
        values??(0??) = &putApplid.paUserid;
        values??(1??) = &aibioarea.regid;
        values??(2??) = NULL;
        values??(3??) = NULL;
        buildReport("Q121",
                    values);
        rc = sendErrorMessage();
        if (rc < RETURN_WARN) {
          rc = RETURN_WARN;
        }
      } /* (strcmp(info->deadQueueName, " ") != 0) */
      else {
        objDesc.ObjectType = MQOT_Q;
        strncpy(objDesc.ObjectName,
                &info->deadQueueName??(0??),
                MQ_Q_NAME_LENGTH);
        openopts = MQOO_INQUIRE +
                   MQOO_OUTPUT +
                   MQOO_PASS_ALL_CONTEXT;
        MQOPEN(info->hconn,
               &objDesc,
               openopts,
               &info->hdeadq,
               &mqcc,
               &mqrc);
        if (mqcc != MQCC_OK) {
          debugLine(DEBUG_LEVEL_MINIMUM,
                    "ERROR Opening dead letter queue for inquiry " \
                    "(mqcc %ld mqrc %ld)",
                    mqcc,
                    mqrc);
          if ((mqrc == MQRC_Q_MGR_STOPPING) ||
              (mqrc == MQRC_Q_MGR_QUIESCING) ||
              (mqrc == MQRC_CONNECTION_BROKEN) ||
              (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
            endingMessage();
            rc = RETURN_STOP;
          } /* mqrc MQOPEN on DLQ */
          else {
            values??(0??) = &putApplid.paUserid;
            values??(1??) = &aibioarea.regid;
            values??(2??) = &mqcc;
            values??(3??) = &mqrc;
            buildReport("Q112",
                        values);
            rc = sendErrorMessage();
            if (rc == RETURN_OK) {
              values??(0??) = &info->deadQueueName;
              values??(1??) = NULL;
              values??(2??) = NULL;
              values??(3??) = NULL;
              buildReport("Q110",
                          values);
              sendErrorMessage();
            }
            rc = RETURN_FAIL;
          } /* else mqrc MQOPEN on DLQ */
        } /* end elseif mqcc != MQCC_OK on MQOPEN on DLQ */
        else {
          debugLine(DEBUG_LEVEL_RETURN_CODES,
                    "Opening dead letter queue for inquiry " \
                    "(mqcc %ld mqrc %ld)",
                    mqcc,
                    mqrc);
          selectors??(0??) = MQIA_MAX_MSG_LENGTH;
          selectors??(1??) = MQIA_USAGE;

          MQINQ(info->hconn,        /* Queue Manager Handle          */
                info->hdeadq,       /* Queue Managr enquiry handle.  */
                2,                  /* Number of selectore           */
                selectors,          /* Selectors                     */
                2,                  /* Number of returned Intergers  */
                alIntAttrs,         /* Returned Integers             */
                0,                  /* Length of returned chars      */
                NULL,               /* Returned chars                */
                &mqcc,              /* Completion code               */
                &mqrc);             /* Reason code                   */

          if (mqcc != MQCC_OK) {
            debugLine(DEBUG_LEVEL_MINIMUM,
                      "ERROR inquirying dead letter queue for " \
                      "(mqcc %ld mqrc %ld)",
                      mqcc,
                      mqrc);
            if ((mqrc == MQRC_Q_MGR_STOPPING) ||
                (mqrc == MQRC_Q_MGR_QUIESCING) ||
                (mqrc == MQRC_CONNECTION_BROKEN) ||
                (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
              endingMessage();
              rc = RETURN_STOP;
            } /* mqrc options on MQINQ DLQ */
            else if (mqrc == MQRC_SELECTOR_NOT_FOR_TYPE) {
              values??(0??) = NULL;
              values??(1??) = NULL;
              values??(2??) = NULL;
              values??(3??) = NULL;
              buildReport("Q123",
                          values);
              rc = sendErrorMessage();
              if (rc == RETURN_OK) {
                values??(0??) = &info->deadQueueName;
                values??(1??) = NULL;
                values??(2??) = NULL;
                values??(3??) = NULL;
                buildReport("Q110",
                            values);
                rc = sendErrorMessage();
                if (rc == RETURN_OK) {
                  rc = RETURN_WARN;
                }
              }
            } /* mqrc == MQRC_SELECTOR_NOT_FOR_TYPE */
            else {
              debugLine(DEBUG_LEVEL_RETURN_CODES,
                        "Inquirying dead letter queue" \
                        "(mqcc %ld mqrc %ld)",
                        mqcc,
                        mqrc);
              values??(0??) = &putApplid.paUserid;
              values??(1??) = &aibioarea.regid;
              values??(2??) = &mqcc;
              values??(3??) = &mqrc;
              buildReport("Q114",
                          values);
              sendErrorMessage();
              values??(0??) = &info->deadQueueName;
              values??(1??) = NULL;
              values??(2??) = NULL;
              values??(3??) = NULL;
              buildReport("Q110",
                          values);
              sendErrorMessage();
              rc = RETURN_FAIL;
            } /* else on mqrc MQINQ on DLQ */
          } /* end if mqcc != MQCC_OK) on QLQ */
          else if (alIntAttrs??(1??) != MQUS_NORMAL) {
            debugLine(DEBUG_LEVEL_MINIMUM,
                      "Dead letter queue use is not normal");
            values??(0??) = &putApplid.paUserid;
            values??(1??) = &aibioarea.regid;
            values??(2??) = NULL;
            values??(3??) = NULL;
            buildReport("Q124",
                        values);
            sendErrorMessage();
            values??(0??) = &info->deadQueueName;
            values??(1??) = NULL;
            values??(2??) = NULL;
            values??(3??) = NULL;
            buildReport("Q110",
                        values);
            sendErrorMessage();
            rc = RETURN_WARN;
          } /* alIntAttrs??(1??) != MQUS_NORMAL */
          else {
            /* MQINQ completed successfully */
            info->deadMaxSize = alIntAttrs??(0??);
          } /* else else mqcc != OK on MQINQ on DLQ */
        } /* end else mqcc != MQCC_OK on MQOPEN on DLQ */
      } /* end else strcmp(info->deadQueueName, " ") != 0 */
    } /* end else mqcc != MQCC_OK on MQINQ on Qmgr */
    closeQueue(info->hconn, hqmgr, QUEUE_MANAGER);
  } /* end else mqcc != MQCC_OK on MQOPEN on Qmgr */
  debugLine(DEBUG_LEVEL_FULL,
            "Gathered dead letter queue information");

  return rc;
}
/*********************************************************************/
/* End of function getDLQInformation                                 */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  getLongMessage                                    */
/*                                                                   */
/* Description: This procedure handles a record on the initation     */
/*              queue which is longer than the MQTM structure.       */
/*              It gets storage, MQGETS the record and builds a      */
/*              message for the dead letter queue.                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info         - Ptr to the MQ infomation.       */
/*                    msgLength    - message length.                 */
/*                    mqmd         - Ptr to the MQMD of the orginal  */
/*                                   MQMD.                           */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short getLongMessage(QMGR_INFO *info,
                     MQTM *msgBuffer,
                     MQLONG *dataLength,
                     MQMD *mqmd) {
  MQLONG       newsize;                 /* Length of MQGET msg +MQDLH*/
                                        /* Buffer length             */
  int const    msgLength = sizeof(MQTM);
  MQGMO        mqgmo = {MQGMO_DEFAULT}; /* Get message options       */
  MQLONG       mqcc = 0;                /* Completion code           */
  MQLONG       mqrc = 0;                /* Reason code               */
  short        rc   = RETURN_OK;        /* Return code from functions*/
  void         *values[4];

  /*******************************************************************/
  /*                                                                 */
  /* Issue error message before putting to DLQ                       */
  /*                                                                 */
  /*******************************************************************/
  debugLine(DEBUG_LEVEL_FULL, "Getting long message");
  values??(0??) = &putApplid.paUserid;
  values??(1??) = &aibioarea.regid;
  values??(2??) = dataLength;
  values??(3??) = NULL;
  buildReport("Q111",
              values);
  rc = sendErrorMessage();

  if (rc == RETURN_OK) {
    /*****************************************************************/
    /*                                                               */
    /* Get some storage for the long message. If storage not         */
    /* availible issue warning and "lose" long message.  ?????       */
    /*                                                               */
    /*****************************************************************/
    newsize = *dataLength;
    free(msgBuffer);
    msgBuffer = (MQTM*)malloc(newsize);
    if (msgBuffer != NULL) {
      /* Get the message */
      mqgmo.Options = MQGMO_NO_SYNCPOINT +
                      MQGMO_FAIL_IF_QUIESCING;
      mqgmo.WaitInterval = 0;
      memset(msgBuffer, ' ', newsize);

      MQGET(info->hconn,               /* Queue manager handle       */
            info->hinitq,              /* Queue handle               */
            mqmd,                      /* Message descriptor         */
            &mqgmo,                    /* Get message options        */
            newsize,                   /* Buffer length              */
            msgBuffer,                 /* Buffer                     */
            dataLength,                /* Data length                */
            &mqcc,                     /* Completion code            */
            &mqrc);                    /* Reason code                */

      /***************************************************************/
      /* Anything other that a shutdown request or a normal          */
      /* completion is serious. There should be enough storage and   */
      /* the duff message should still be there.                     */
      /***************************************************************/
      if (mqcc == MQCC_OK) {
        debugLine(DEBUG_LEVEL_RETURN_CODES,
                   "Getting long message (mqcc %ld mqrc %ld)",
                   mqcc,
                   mqrc);
        /* MQGET completed successfully */
        rc = putToDLQ(info,
                      msgBuffer,
                      *dataLength,
                      MQFB_TM_ERROR);
      } /* end if (mqcc == MQCC_OK) */
      else {
        /* MQGET DID NOT complete successfully */
        debugLine(DEBUG_LEVEL_RETURN_CODES,
                   "Error getting long message (mqcc %ld mqrc %ld)",
                   mqcc,
                   mqrc);
        if ((mqrc == MQRC_Q_MGR_STOPPING) ||
            (mqrc == MQRC_Q_MGR_QUIESCING) ||
            (mqrc == MQRC_CONNECTION_BROKEN) ||
            (mqrc == MQRC_CONNECTION_STOPPING) ||
            (mqrc == MQRC_CONNECTION_QUIESCING)) {
          endingMessage();
          rc = RETURN_STOP;
        }
        else {
          values??(0??) = &putApplid.paUserid;
          values??(1??) = &aibioarea.regid;
          values??(2??) = &mqcc;
          values??(3??) = &mqrc;
          buildReport("Q106",
                      values);
          sendErrorMessage();
          rc = RETURN_FAIL;
        }
      } /* end if (msgBuffer != NULL) */
      /* else memory error */
    } /* end else */
  } /* end if rc == RETUTN_OK */

  return rc;
}
/*********************************************************************/
/* End of function getLongMessage                                    */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  getMessage                                        */
/*                                                                   */
/* Description: MQGET a message from the initiation queue.           */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info         - Ptr to the MQ infomation        */
/*                                                                   */
/* Output Parameters: msgLength    - Ptr to message length.          */
/*                    msgBuffer    - Ptr to the message.             */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_WARN      - if there is no message to   */
/*                                       get.                        */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                                                                   */
/*********************************************************************/
short getMessage(QMGR_INFO *info,
                 MQTM *databuff,
                 MQLONG *dataLength) {
  MQLONG       lDlhDataLength;          /* Length of MQGET msg +MQDLH*/
                                        /* Buffer length             */
  MQGMO        mqgmo = {MQGMO_DEFAULT}; /* Get message options       */
  MQMD         mqmd = {MQMD_DEFAULT};   /* Message descriptor        */
  short        fStartTrans = TRUE;      /* Flag to start transaction */
  MQLONG       mqcc = 0;                /* Completion code           */
  MQLONG       mqrc = 0;                /* Reason code               */
  short        rc   = RETURN_OK;        /* Return code from functions*/
  int i;
  void *values[4];

  memcpy(mqmd.StrucId, MQMD_STRUC_ID, sizeof(mqmd.StrucId));
  mqmd.Version  = MQMD_VERSION_1;

  /* Get the message */
  memcpy(mqgmo.StrucId, MQGMO_STRUC_ID, sizeof(mqgmo.StrucId));
  mqgmo.Version  = MQGMO_VERSION_1;
  mqgmo.Options  = MQGMO_SYNCPOINT +
                   MQGMO_FAIL_IF_QUIESCING +
                   MQGMO_WAIT;
  mqgmo.WaitInterval = WAITINTERVAL;
  memset(databuff, '\0', sizeof(MQTM));

  debugLine(DEBUG_LEVEL_FULL,
             "Getting message, will wait for %d seconds",
             (WAITINTERVAL / 1000));
  MQGET(info->hconn,                   /* Queue manager handle       */
        info->hinitq,                  /* Queue handle               */
        &mqmd,                         /* Message descriptor         */
        &mqgmo,                        /* Get message options        */
        sizeof(MQTM),                  /* Buffer length              */
        databuff,                      /* Buffer                     */
        dataLength,                    /* Data length                */
        &mqcc,                         /* Completion code            */
        &mqrc);                        /* Reason code                */

  if (mqcc == MQCC_OK) {
    /* MQGET completed successfully */
    debugLine(DEBUG_LEVEL_RETURN_CODES,
               "Getting message (mqcc %ld mqrc %ld)",
               mqcc,
               mqrc);
    if (*dataLength != sizeof(MQTM)) { /* Message must be shorter.   */
      /***************************************************************/
      /*                                                             */
      /* Issue error message before putting to DLQ                   */
      /*                                                             */
      /***************************************************************/
      debugLine(DEBUG_LEVEL_FULL,
                "Sizes don't match - dataLength = %ld, " \
                "sizeof(MQTM) = %ld",
                *dataLength,
                sizeof(MQTM));
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = dataLength;
      values??(3??) = NULL;
      buildReport("Q111",
                  values);
      rc = sendErrorMessage();

      rc = putToDLQ(info,
                    databuff,
                    *dataLength,
                    MQFB_TM_ERROR);
    }
  } /* end if (mqcc == MQCC_OK) */
  else {
    /* MQGET DID NOT complete successfully */
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR Getting message (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    if (mqrc == MQRC_NO_MSG_AVAILABLE) {
      rc = RETURN_WARN;
    }
    else if ((mqrc == MQRC_Q_MGR_STOPPING) ||
             (mqrc == MQRC_Q_MGR_QUIESCING) ||
             (mqrc == MQRC_CONNECTION_BROKEN) ||
             (mqrc == MQRC_CONNECTION_STOPPING) ||
             (mqrc == MQRC_CONNECTION_QUIESCING)) {
      endingMessage();
      rc = RETURN_STOP;
    }
    else if (mqrc == MQRC_TRUNCATED_MSG_FAILED) {
      rc = getLongMessage(info,
                          databuff,
                          dataLength,
                          &mqmd);
    }
    else {
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q106",
                  values);
      rc = sendErrorMessage();
      if (rc != RETURN_OK) {
        values??(0??) = &info->initQueueName;
        values??(1??) = NULL;
        values??(2??) = NULL;
        values??(3??) = NULL;
        buildReport("Q110",
                    values);
        rc = sendErrorMessage();
        if (rc == RETURN_OK) {
          rc = RETURN_STOP;
        }
      }
      if (mqrc == MQRC_GET_INHIBITED) {
        rc = RETURN_FAIL;
      }
      /***************************************************************/
      /*                                                             */
      /* If rc is still RETURN_OK but there was an error in the      */
      /* MQGET we must notify the user.                              */
      /*                                                             */
      /***************************************************************/
      if (rc == RETURN_OK) {
        rc = RETURN_STOP;
      }
    }
  } /* end else */

  return rc;
}
/*********************************************************************/
/* End of function getMessage                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: getParams                                          */
/*                                                                   */
/* Description: Open the CSQQUT1 and CSQQUT2 files read the CSQQUT1  */
/*              line by line and parses them and validates the input.*/
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  params       - ptr to a structure to hold the  */
/*                                   input parameters.               */
/*                                                                   */
/* Returns:           RETURN_OK        - if the CSQQUT1 file was     */
/*                                       successfully read and       */
/*                                       parsed.                     */
/*                    RETURN_STOP      - Otherwise.                  */
/*                                                                   */
/*********************************************************************/
short getParams(INPUT_PARAMS *params) {
  FILE *csqqut1 = NULL;
  char buffer??(100??);
  short rc = RETURN_OK;
  short loop = 0;

  debugLine(DEBUG_LEVEL_FULL, "Reading parameters");
  csqqut1 = fopen("DD:CSQQUT1", "r");
  if (csqqut1 == NULL) {
    debugLine(DEBUG_LEVEL_MINIMUM, "Can't open CSQQUT1 file.");
    reportBadOpen();
    rc = RETURN_FAIL;
  }
  else {
    csqqut2 = fopen("DD:CSQQUT2", "w");
    if (csqqut2 != NULL) {
      debugLine(DEBUG_LEVEL_MINIMUM, "Can't open CSQQUT2 file.");
    }
    else {
      debugLine(DEBUG_LEVEL_FULL, "Openned CSQQUT2 file ");
      UT2Headers();
    }
    fgets(buffer, CSQQUT1_BUFFER_LENGTH, csqqut1);
    if (!feof(csqqut1)) {
      do {
        debugLine(DEBUG_LEVEL_MINIMUM,
                  "Read line %s", 
                  buffer);
        for (loop = 0; loop < CSQQUT1_BUFFER_LENGTH; loop++) {
          buffer??(loop??) = toupper(buffer??(loop??));
        }
        if ((buffer??(0??) != '*') &&
            (inputParse(buffer, params) != 0)) {
          showExpectedCSQQUT1();
          rc = RETURN_FAIL;
        }      
        fgets(buffer, CSQQUT1_BUFFER_LENGTH, csqqut1);
      } while ((!feof(csqqut1)) && (rc == RETURN_OK));
    }
    fclose(csqqut1);
    debugLine(DEBUG_LEVEL_FULL, "About to validate");
    if (rc == RETURN_OK) {
      rc = validateInput(params);
      finalAnalysis(params);
    }
  }
  debugLine(DEBUG_LEVEL_FULL, "Leaving getParams()");

  return rc;
}
/*********************************************************************/
/* End of function getParams                                         */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: initialise                                         */
/*                                                                   */
/* Description: Initialise the IMS structures and do a INQY.         */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void initialise() {
  int ctdliRC = 0;

  ims_buffer.ll = sizeof(MQTMC2);       /* Length of message         */
  ims_buffer.zz = 0;                    /* Required by IMS           */

  /*******************************************************************/
  /*  Get IMS region for messages.                                   */
  /*******************************************************************/

  memcpy(aib.id, "DFSAIB  ", sizeof(aib.id));
  aib.len = sizeof(AIB);
  memcpy(aib.sfunc, "ENVIRON ", sizeof(aib.sfunc));
  memcpy(aib.rsnm1, "IOPCB   ", sizeof(aib.rsnm1));
  aib.oalen = sizeof(AIB_IO_AREA);
  aib.oause = 0;

  strncpy(aibioarea.psbName, "CSQ4TRMN", sizeof(psbName));
  params.debugLevel = DEBUG_LEVEL_DEFAULT;
  params.csqqut3Handle = fopen("DD:CSQQUT3", "w");
  if (params.csqqut3Handle == NULL) {
    printf("Can't open CSQQUT3, disabled debug.\n");
    params.debugLevel = DEBUG_FILE_ERROR;
  }
  else {
    debugLine(DEBUG_LEVEL_FULL, "File CSQQUT3 opened for output");
  }
}
/*********************************************************************/
/* End of function initialise                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: inputParse                                         */
/*                                                                   */
/* Description: Parses a line and places the value(s) in the         */
/*              appropriate elements in tthe INPUT_PARAMS structure. */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  buf              - the buffer to parse.        */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  params           - ptr to a structure to hold  */
/*                                       the input parameters.       */
/*                                                                   */
/* Returns:           RETURN_OK        - if the parse found          */
/*                                       recognised parameters       */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short inputParse(char *buf,
                 INPUT_PARAMS *params) {
  char key??(sizeof(MQCHAR48)??);
  char value??(sizeof(MQCHAR48)??);
  char reportbuffer??(REPORT_LINE_LENGTH??);
  char *ptr = buf;
  char *ptrkey;
  char *ptrvalue;
  short rc = RETURN_OK;
  short buflength = CSQQUT1_BUFFER_LENGTH;
  short fieldlength;
  short vtoolong = 0;

  debugLine(DEBUG_LEVEL_FULL, "Parsing line '%s'", buf);
  memset(key, '\0', sizeof(MQCHAR48));
  memset(value, '\0', sizeof(MQCHAR48));
  memset(value, ' ', 8);
  while ((isspace(*ptr)) && (buflength != 0)) {
    ptr += sizeof(char);
    buflength--;
  }
  if (buflength == 0) {
    return;
  }
  ptrkey = &key??(0??);
  fieldlength = sizeof(MQCHAR48);
  while ((*ptr != '=')  && (buflength != 0) && (!isspace(*ptr))) {
    if (fieldlength > 0) {
      *ptrkey = *ptr;
      ptr += sizeof(char);
      ptrkey += sizeof(char);
      buflength--;
    }
    else {
      debugLine(DEBUG_LEVEL_MINIMUM, 
                "ERROR: Key %s is to long", 
                key);
      rc = 1;
      buflength = 0;  /* Terminate loop */
    }
    fieldlength--;
  }
  ptr += sizeof(char); /* Skip over the equals. */
  ptrvalue = &value??(0??);
  fieldlength = sizeof(MQCHAR48);
  while ((!isspace(*ptr))  && (buflength != 0)) {
    if (fieldlength > 0) {
      *ptrvalue = *ptr;
      ptr += sizeof(char);
      ptrvalue += sizeof(char);
      buflength--;
    }
    else {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "ERROR: Value for key %s is to long",
                key);
      vtoolong = 1;
      buflength = 0;  /* Terminate loop */
    }
    fieldlength--;
  }
  memset(reportbuffer, '\0', REPORT_LINE_LENGTH);
  sprintf(reportbuffer, "%s=%s", key, value);
  if (vtoolong == 1) {
    reportIt(reportbuffer, FALSE);
    rc = RETURN_FAIL;
  }
  else if (strcmp(key, "QMGRNAME") == 0) {
    strncpy(params->qmgrName, value, sizeof(MQCHAR48));
    reportIt(reportbuffer, TRUE);
  }
  else if (strcmp(key, "INITQUEUENAME") == 0) {
    strncpy(params->initQueueName, value, sizeof(MQCHAR48));
    reportIt(reportbuffer, TRUE);
  }
  else if (strcmp(key, "LTERM") == 0) {
    memcpy(params->lterm, value, 8);
    reportIt(reportbuffer, TRUE);
  }
  else if (strcmp(key, "CONSOLEMESSAGES") == 0) {
    strncpy(params->consoleMessagesText, value, sizeof(MQCHAR48));
    reportIt(reportbuffer, TRUE);
  }
  else if (strcmp(key, "DEBUGLEVEL") == 0) {
    if (params->debugLevel != DEBUG_FILE_ERROR) {
      params->debugLevel = atoi(value);
      reportIt(reportbuffer, TRUE);
    }
  }
  else {
    reportIt(reportbuffer, FALSE);
    rc = RETURN_FAIL;
  }

  return rc;
}
/*********************************************************************/
/* End of function inputParse                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: openDeadQueue                                      */
/*                                                                   */
/* Description: MQOPEN the dead letter queue.                        */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none.                                          */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  info             - Ptr to the MQ infomation    */
/*                                                                   */
/* Returns:           RETURN_OK        - if the dead letter queue    */
/*                                       queue opened.               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short openDeadQueue(QMGR_INFO *info) {
  MQOD      objDesc = {MQOD_DEFAULT};  /* Object descriptor          */
  MQLONG    openopts;                  /* Open options               */
  MQLONG    mqcc = 0;                  /* Completion code            */
  MQLONG    mqrc = 0;                  /* Reason code                */
  short     rc = RETURN_OK;
  void      *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Opening dead letter queue '%s' on '%s'",
            info->deadQueueName,
            info->qmgrName);
  objDesc.ObjectType = MQOT_Q;
  strncpy(objDesc.ObjectName,
          &info->deadQueueName??(0??),
          MQ_Q_NAME_LENGTH);
  openopts = MQOO_INQUIRE +
             MQOO_OUTPUT +
             MQOO_PASS_ALL_CONTEXT;
  MQOPEN(info->hconn,
         &objDesc,
         openopts,
         &info->hdeadq,
         &mqcc,
         &mqrc);
  if (mqcc != MQCC_OK) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR: Opening dead letter (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    if ((mqrc == MQRC_Q_MGR_STOPPING) ||
        (mqrc == MQRC_Q_MGR_QUIESCING) ||
        (mqrc == MQRC_CONNECTION_BROKEN) ||
        (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
      endingMessage();
      rc = RETURN_STOP;
    } /* mqrc MQOPEN on DLQ */
    else {
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q112",
                  values);
      rc = sendErrorMessage();
      if (rc == RETURN_OK) {
        values??(0??) = &info->deadQueueName;
        values??(1??) = NULL;
        values??(2??) = NULL;
        values??(3??) = NULL;
        buildReport("Q110",
                    values);
        sendErrorMessage();
      }
      rc = RETURN_FAIL;
    } /* end else mqrc MQOPEN on DLQ */
  } /* end if (mqcc != MQCC_OK) */
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Opening dead letter (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
  }

  return rc;
}
/*********************************************************************/
/* End of function openDeadQueue                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: openInitiationQueue                                */
/*                                                                   */
/* Description: MQOPEN the initiation queue.                         */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none.                                          */
/*                                                                   */
/* Output Parameters: none.                                          */
/*                                                                   */
/* InOut Parameters:  info             - Ptr to the MQ infomation    */
/*                                                                   */
/* Returns:           RETURN_OK        - if the initiation queue was */
/*                                       opened.                     */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short openInitiationQueue(QMGR_INFO *info) {

  MQOD      objDesc = {MQOD_DEFAULT};  /* Object descriptor          */
  MQLONG    openOptions;               /* Open options               */
  MQLONG    mqcc = 0;                  /* Completion code            */
  MQLONG    mqrc = 0;                  /* Reason code                */
  short     rc = RETURN_OK;
  void      *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Opening initiation queue '%s' on '%s'",
            info->initQueueName,
            info->qmgrName);
  /* MQOPEN the initiation queue */
  strncpy(objDesc.ObjectName,
          &info->initQueueName??(0??),
          MQ_Q_NAME_LENGTH);
  openOptions = MQOO_INPUT_AS_Q_DEF +
                MQOO_FAIL_IF_QUIESCING +
                MQOO_SAVE_ALL_CONTEXT;

  MQOPEN(info->hconn,
         &objDesc,                    /* Object descriptor           */
         openOptions,                 /* Open options                */
         &info->hinitq,               /* Initiation queue handle     */
         &mqcc,                       /* Completion code             */
         &mqrc);                      /* Reason code                 */

  if (mqcc != MQCC_OK) {
    /* MQOPEN was NOT successful */
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR: Opening initiation (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    if ((mqrc == MQRC_Q_MGR_STOPPING) ||
        (mqrc == MQRC_Q_MGR_QUIESCING) ||
        (mqrc == MQRC_CONNECTION_BROKEN) ||
        (mqrc == MQRC_CONNECTION_STOPPING) ||
        (mqrc == MQRC_CONNECTION_QUIESCING)) {
      endingMessage();
      rc = RETURN_STOP;
    }
    else {
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q101",
                  values);
      sendErrorMessage();
      rc = RETURN_FAIL;
    }
  } /* end if (mqcc != MQCC_OK) */
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Opening initiation (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
  }

  return rc;
}
/*********************************************************************/
/* End of function openDeadQueue                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  processMQTM                                       */
/*                                                                   */
/* Description: process the MQTM header.                             */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info             - Ptr to the MQ infomation.   */
/*                    msgLength        - message length.             */
/*                    msgBuffer        - Ptr to the message.         */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short processMQTM(QMGR_INFO *info,
                  MQTM  *msgBuffer,
                  MQLONG msgLength) {
  MQLONG dlh_reason = 0;
  short startTransaction = TRUE;
  short rc = RETURN_OK;

  debugLine(DEBUG_LEVEL_FULL, "Processing MQTM");
  if (msgLength == sizeof(MQTM)) {
    /*Message is the correct length */

    /* Check the structure */
    if (strncmp(msgBuffer->StrucId,
                MQTM_STRUC_ID,
                sizeof(msgBuffer->StrucId)) != 0) {
      debugLine(DEBUG_LEVEL_FULL,
                "StrucIds don't match '%s' != '%s'",
                msgBuffer->StrucId,
                MQTM_STRUC_ID);
      dlh_reason = MQFB_TM_ERROR;
      startTransaction = FALSE;
    }
    /* Check the version */
    else if (msgBuffer->Version != MQTM_VERSION_1) {
      debugLine(DEBUG_LEVEL_FULL,
                "Versions don't match %ld != %ld",
                msgBuffer->Version,
                MQTM_VERSION_1);
      dlh_reason = MQFB_TM_ERROR;
      startTransaction = FALSE;
    }
    /* Check the application type */
    else if (msgBuffer->ApplType != MQAT_IMS) {
      debugLine(DEBUG_LEVEL_FULL,
                "ApplType must be IMS(%d) and not %d",
                MQAT_IMS,
                msgBuffer->ApplType);
      dlh_reason = MQFB_APPL_TYPE_ERROR;
      startTransaction = FALSE;
    }
  } /* end if (msgLength == sizeof(MQTM)) */
  else {
    startTransaction = FALSE;
  }

  if (startTransaction == TRUE) {
    rc = startTrans(info,
                    msgBuffer,
                    msgLength);
  }
  else {
    rc = putToDLQ(info,
                  msgBuffer,
                  msgLength,
                  dlh_reason);
  }

  return rc;
}
/*********************************************************************/
/* End of function processMQTM                                       */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  putMsgToDLQ                                       */
/*                                                                   */
/* Description: put message to dead letter queue information.        */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info             - Ptr to the MQ infomation.   */
/*                    msgLength        - message length.             */
/*                    msgBuffer        - Ptr to the message.         */
/*                    dlqReason        - DL reason.                  */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short putMsgToDLQ(QMGR_INFO *info,
                  char      *msgBuffer,
                  MQLONG    msgBufferLen,
                  MQLONG    dlqReason) {
  MQMD   mqmd = {MQMD_DEFAULT};        /* Message descriptor         */
  MQPMO  mqpmo = {MQPMO_DEFAULT};      /* Put message Options        */
  MQDLH  mqdlh = {MQDLH_DEFAULT};      /* Dead letter header         */
  MQBYTE *newBuffer;
  MQLONG newBufferLen;
  MQLONG ltoCopy;
  MQLONG mqcc = 0;                     /* Completion code            */
  MQLONG mqrc = 0;                     /* Reason code                */
  char   csAbstime??(8??)  ;           /* Packed decimal date/time   */
  char   csTime??(8??);                /* CICS Time                  */
  char   csDate??(8??);                /* CICS Date                  */
  short  rc = DLQED;
  void   *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Putting message to dead letter queue");
  debugLine(DEBUG_LEVEL_FULL,
            "DLQ reason %d",
            dlqReason);
  mqmd.Encoding       = MQENC_NATIVE;
  mqmd.CodedCharSetId = MQCCSI_Q_MGR;
  memcpy(mqmd.Format, MQFMT_DEAD_LETTER_HEADER, sizeof(mqmd.Format));

  /* Create dead letter header */
  memcpy(mqdlh.StrucId, MQDLH_STRUC_ID, sizeof(mqdlh.StrucId));
  mqdlh.Version = MQDLH_VERSION_1;
  mqdlh.Reason  = dlqReason;
  memcpy(mqdlh.DestQName,
         (char *)info->initQueueName,
         MQ_Q_NAME_LENGTH);
  memcpy(mqdlh.DestQMgrName,
         (char *)info->qmgrName,
         MQ_Q_MGR_NAME_LENGTH);
  mqdlh.Encoding = mqmd.Encoding;
  mqdlh.CodedCharSetId = mqmd.CodedCharSetId;
  memcpy(mqdlh.Format,
         mqmd.Format,
         sizeof(mqdlh.Format));
  mqdlh.PutApplType = MQAT_IMS;

  /* Get the GMT time and date */
  memset(csDate,'\0',8);
  memset(csTime,'\0',8);
  memcpy(mqdlh.PutDate, &csDate, sizeof(mqdlh.PutDate));
  memcpy(mqdlh.PutTime, &csTime, sizeof(mqdlh.PutTime));

  /* Create the message descriptor */
  strncpy(mqmd.Format,
          MQFMT_DEAD_LETTER_HEADER,
          sizeof(mqmd.Format));
  /* Create the put message options */
  memcpy(mqpmo.StrucId, MQPMO_STRUC_ID, sizeof(mqpmo.StrucId));
  mqpmo.Version         = MQPMO_VERSION_1;
  mqpmo.Options         = MQPMO_NO_SYNCPOINT +
                          MQPMO_PASS_ALL_CONTEXT;
  mqpmo.Context         = info->hinitq;

  /* Malloc the storage required. */
  newBufferLen = msgBufferLen + sizeof(MQDLH);
  ltoCopy = msgBufferLen;

  if (newBufferLen > info->deadMaxSize)  {
    newBufferLen = info->deadMaxSize;
    ltoCopy = info->deadMaxSize - sizeof(MQDLH);
  }
  newBuffer = (MQBYTE *)malloc(newBufferLen);
  /* Add the dead letter header to the message buffer */
  memcpy(newBuffer,
         &mqdlh,
         sizeof(MQDLH));
  memcpy(newBuffer + sizeof(MQDLH),
         msgBuffer,
         ltoCopy);

  /* Put the message to the dead letter queue */
  MQPUT(info->hconn,
        info->hdeadq,                  /* Dead letter queue handle   */
        &mqmd,                         /* Message descriptor         */
        &mqpmo,                        /* Put message options        */
        newBufferLen,                  /* Buffer length              */
        newBuffer,                     /* Buffer                     */
        &mqcc,                         /* Completion code            */
        &mqrc);                        /* Reason code                */
  if (mqcc != MQCC_OK) {
    /* MQPUT was NOT successful */
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR: putting to DLQ (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
    if ((mqrc == MQRC_Q_MGR_STOPPING) ||
        (mqrc == MQRC_Q_MGR_QUIESCING) ||
        (mqrc == MQRC_CONNECTION_BROKEN) ||
        (mqrc == MQRC_Q_MGR_NOT_AVAILABLE)) {
      endingMessage();
      rc = RETURN_STOP;
    }
    else {
      values??(0??) = &putApplid.paUserid;
      values??(1??) = &aibioarea.regid;
      values??(2??) = &mqcc;
      values??(3??) = &mqrc;
      buildReport("Q120",
                  values);
      rc = sendErrorMessage();
      if (rc == RETURN_OK) {
        values??(0??) = &info->deadQueueName;
        values??(1??) = NULL;
        values??(2??) = NULL;
        values??(3??) = NULL;
        buildReport("Q110",
                    values);
        sendErrorMessage();
      }
      rc = RETURN_FAIL;
    } /* end if mqrc */
    values??(0??) = &putApplid.paUserid;
    values??(1??) = &aibioarea.regid;
    values??(2??) = NULL;
    values??(3??) = NULL;
    buildReport("Q113",
                values);
    rc = sendErrorMessage();
    if (rc == RETURN_OK) {
      values??(0??) = &mqmd.MsgId;
      values??(1??) = NULL;
      values??(2??) = NULL;
      values??(3??) = NULL;
      buildReport("Q118",
                  values);
      rc = sendErrorMessage();
    }
  } /* end if mqcc != MQCC_OK) */
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "Putting to DLQ (mqcc %ld mqrc %ld)",
              mqcc,
              mqrc);
  }
  free(newBuffer);

  return rc;
}
/*********************************************************************/
/* End of function putMsgToDLQ                                       */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: putToDLQ                                           */
/*                                                                   */
/* Description: Calls procedures to open the dead letter queue       */
/*              and add a message to it.                             */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  msgLength        - message length.             */
/*                    msgBuffer        - Ptr to the message.         */
/*                    dlqReason        - DL reason.                  */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  info             - Ptr to the MQ infomation.   */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short putToDLQ(QMGR_INFO *info,
               MQTM *msgBuffer,
               MQLONG msgLength,
               MQLONG dlReason) {
  short rc = DLQED;                /* Return code from functions     */
  short temprc;
  void  *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Preparing to put message to dead letter queue");
  /* Get the dead letter queue name and open if we haven't got that */
  /* information already.                                           */
  if (info->deadMaxSize == -1) {
    debugLine(DEBUG_LEVEL_FULL,
              "Dead letter queue information required");
    rc = getDLQInformation(info);
  }
  if ((rc == RETURN_OK) ||
      (rc == DLQED)) {
    if (info->hdeadq == -1) {
      debugLine(DEBUG_LEVEL_FULL,
                "Need to open dead letter queue");
      rc = openDeadQueue(info);
    }
    if (rc == 0) {
      /* Put to dead letter queue */
      rc = putMsgToDLQ(info,
                       (char *)msgBuffer,
                       msgLength,
                       dlReason);
    } /* end if (rc == 0) */
    /* Close the dead letter queue */
    debugLine(DEBUG_LEVEL_FULL,
              "Preparing close to dead letter queue");
    temprc = closeQueue(info->hconn, info->hdeadq, DEAD_LETTER_QUEUE);
    if (temprc > 0) { /* Error has occured */
      values??(0??) = &info->deadQueueName;
      values??(1??) = NULL;
      values??(2??) = NULL;
      values??(3??) = NULL;
      buildReport("Q110",
                  values);
      sendErrorMessage();
      rc = RETURN_FAIL;
    }
    else if (temprc > rc) {
      rc = temprc;
    }
    info->hdeadq = -1;
  }

  return rc;
}
/*********************************************************************/
/* End of function putToDLQ                                          */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: reportBadOpen                                      */
/*                                                                   */
/* Description: Report the fact that we couldn't open the CSQQUT1    */
/*              file.                                                */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void reportBadOpen() {
  void *values[4];

  values??(0??) = MODULE_NAME;
  values??(1??) = NULL;
  values??(2??) = NULL;
  values??(3??) = NULL;
  buildReport("Q152",
              values);
  writeLine(csqqut2);
}
/*********************************************************************/
/* End of function reportBadOpen                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: reportIt                                           */
/*                                                                   */
/* Description: Reflect input record onto CSQQUT2 and say whether or */
/*              not it is accepted or not.                           */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  buff         - the input buffer                */
/*                    accepted     - whether the input was accepted  */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void reportIt(char *buff,
              int accepted) {
  void *values[4];

  strncpy(report.reportline, buff, REPORT_LINE_LENGTH);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, "%s accepted %d", buff, accepted);

  if (accepted == FALSE) {
    values??(0??) = MODULE_NAME;
    values??(1??) = NULL;
    values??(2??) = NULL;
    values??(3??) = NULL;
    buildReport("Q153",
                values);
    writeLine(csqqut2);
  }
}
/*********************************************************************/
/* End of function reportIt                                          */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: sendErrorMessage                                   */
/*                                                                   */
/* Description: Send the error message to the IMS message PCB.       */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
short sendErrorMessage() {
  MQCHAR48 oldlterm;
  REPORT_LINE toIMS;
  int ctdliRC = 0;
  short usedefaultlterm = FALSE;
  short loopc;
  short numsecs;
  short rc = RETURN_OK;
  void *values[4];

  memset(&toIMS, '\0', sizeof(REPORT_LINE));
  parmCount = 3;
  ctdliRC = CTDLI(&parmCount, CHNG, msgPCB, &params.lterm);
  if (ctdliRC != 0) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "%x returned on CHNG command using CTDLI()",
              ctdliRC);
    if (memcmp(params.lterm, DEFAULT_LTERM, 8) != TRUE) {
      usedefaultlterm = TRUE;
      memcpy(oldlterm, params.lterm, 9);
      memcpy(params.lterm, DEFAULT_LTERM, 9);
      rc = RETURN_FAIL;
    }
    else if (params.consoleMessages == 1) { /* We're snookered. */
      parmCount = 1;
      ctdliRC = CTDLI(&parmCount, ROLL);
      debugLine(DEBUG_LEVEL_MINIMUM,
                "%x returned PURG command using CTDLI()",
                ctdliRC);
      return;
    }
    debugLine(DEBUG_LEVEL_MINIMUM,
              "%x returned on CHNG command using CTDLI()",
              ctdliRC);
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "%x returned on CHNG command using CTDLI()",
              ctdliRC);
  }
  /*******************************************************************/
  /* For (possibly) multiply message segments ISRT segment to msgPCB */
  /*                                                                 */
  /* If ISPF fails abend with IMS ROLL call                          */
  /*******************************************************************/

  numsecs = (report.ll / 80) + 1;
  for (loopc = 0; loopc < numsecs; loopc++) {
    if (params.consoleMessages == 0) {
      strncpy(toIMS.msgBuffer,
              &report.reportline??(80 * loopc??),
              80);
    }
    toIMS.ll = strlen(toIMS.msgBuffer) + (sizeof(short int) * 2);
    toIMS.zz = 0;

    parmCount = 3;
    ctdliRC = CTDLI(&parmCount, ISRT, msgPCB, &toIMS);
    if (ctdliRC != 0) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "%x returned on ISRT command using CTDLI()",
                ctdliRC);
      parmCount = 1;
      ctdliRC = CTDLI(&parmCount, ROLL);
      debugLine(DEBUG_LEVEL_RETURN_CODES,
                "%x returned on ROLL command using CTDLI()",
                ctdliRC);
    }
    else {
      debugLine(DEBUG_LEVEL_RETURN_CODES,
                "%x returned on ISRT command using CTDLI()",
                ctdliRC);
    }
    memset(&toIMS.msgBuffer, '\0', sizeof(toIMS.msgBuffer));
  }
  if (usedefaultlterm == TRUE) {
    values??(0??) = &putApplid.paUserid;
    values??(2??) = &aibioarea.regid;
    values??(2??) = &oldlterm;
    values??(3??) = NULL;
    buildReport("Q108",
                values);
    rc = sendErrorMessage();
  }
  parmCount = 2;
  ctdliRC = CTDLI(&parmCount, PURG, msgPCB);
  if (ctdliRC != 0) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "%x returned on PURG command using CTDLI()",
              ctdliRC);
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "%x returned on PURG command using CTDLI()",
              ctdliRC);
  }

  /* Empty for next message.                                         */
  memset(&report, '\0', sizeof(ERROR_MESSAGE_AREA));

  return rc;
}
/*********************************************************************/
/* End of function sendErrorMessage                                  */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  showExpectedCSQQUT1                               */
/*                                                                   */
/* Description: Shows the expected input/format of the input file    */
/*              CSQQUT1.                                             */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void showExpectedCSQQUT1()
{
  if (params.debugLevel == DEBUG_FILE_ERROR) {
    printf("The required format for CSQQUT1 is:-\n");
    printf("\n");
    printf("QMGRNAME=q_manager_name        " \
                 "Comment: queue manager name\n");
    printf("INITQUEUENAME=init_q_name      " \
                 "Comment: initiation queue name\n");
    printf("LTERM=lterm                    " \
                 "Comment: LTERM to remove error messages\n");
    printf("CONSOLEMESSAGES=YES            " \
                 "Comment: Send error messages to console\n");
    printf("DEBUGLEVEL=debuglevel          " \
                 "Comment: Debug level to use (0 to 4)\n");
    printf("\n");
    printf("Please refer to the System Administration Guide\n");
    printf("for further information.\n");
  }
  else {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "The required format for CSQQUT1 is:-");
    debugLine(DEBUG_LEVEL_MINIMUM, "");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "QMGRNAME=q_manager_name        " \
              "Comment: queue manager name");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "INITQUEUENAME=init_q_name      " \
              "Comment: initiation queue name");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "LTERM=lterm                    " \
              "Comment: LTERM to remove error messages");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "CONSOLEMESSAGES=YES            " \
              "Comment: Send error messages to console");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "DEBUGLEVEL=debuglevel          " \
              "Comment: Debug level to use (0 to 4)\n");
    debugLine(DEBUG_LEVEL_MINIMUM, "");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "Please refer to the System Administration Guide");
    debugLine(DEBUG_LEVEL_MINIMUM,
              "for further information.");
  }
}
/*********************************************************************/
/* End of function sendErrorReport                                   */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: startTrans                                         */
/*                                                                   */
/* Description: Puts the trigger message to IMS.                     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  info             - Ptr to the MQ infomation.   */
/*                    msgLength        - message length.             */
/*                    msgBuffer        - Ptr to the message.         */
/*                    dlqReason        - DL reason.                  */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_STOP      - If the program should       */
/*                                       terminate normally.         */
/*                    RETURN_WARN      - If the program should       */
/*                                       terminate with a warning.   */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short startTrans(QMGR_INFO *info,
                 MQTM      *msgBuffer,
                 MQLONG    msgBufferLen) {
  IMS_BUFFER ims_buffer;
  MQTMC2 mqtmc2;
  int ctdliRC = 0;
  short rc = RETURN_OK;
  void *values[4];

  debugLine(DEBUG_LEVEL_FULL,
            "Starting transaction with IMS");
  ims_buffer.ll = sizeof(IMS_BUFFER);
  ims_buffer.zz = 0;
  memcpy(&mqtmc2, msgBuffer, sizeof(MQTM));
  memcpy(mqtmc2.StrucId, MQTMC_STRUC_ID, sizeof(MQTMC_STRUC_ID));
  memcpy(mqtmc2.Version, MQTMC_VERSION_2, sizeof(MQTMC_VERSION_2));
  memset(mqtmc2.ApplType, ' ', sizeof(mqtmc2.ApplType));

  debugLine(DEBUG_LEVEL_FULL, "Changing to transPCB");
  parmCount = 3;
  ctdliRC = CTDLI(&parmCount, CHNG, transPCB, &msgBuffer->ApplId);
  if (ctdliRC != 0) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "%x returned on CHNG command using CTDLI()",
              ctdliRC);
    values??(0??) = MODULE_NAME;
    values??(1??) = &CHNG;
    values??(2??) = &ctdliRC;
    values??(3??) = NULL;
    buildReport("Q102",
                values);
    rc = sendErrorMessage();
  }
  else {
    debugLine(DEBUG_LEVEL_RETURN_CODES,
              "%x returned on CHNG command using CTDLI()",
              ctdliRC);
    /*****************************************************************/
    /*                                                               */
    /* We have TRANPCB prepared for IRST call.                       */
    /*                                                               */
    /*****************************************************************/
    memcpy(&ims_buffer.msg, &mqtmc2, sizeof(MQTMC2));
    parmCount = 3;
    ctdliRC = CTDLI(&parmCount, ISRT, transPCB, &ims_buffer);
    if (ctdliRC != 0) {
      debugLine(DEBUG_LEVEL_MINIMUM,
                "%x returned on ISRT command using CTDLI()",
                ctdliRC);
      values??(0??) = MODULE_NAME;
      values??(1??) = &ISRT;
      values??(2??) = &ctdliRC;
      values??(3??) = NULL;
      buildReport("Q102",
                  values);
      sendErrorMessage();
      rc = RETURN_WARN;
    }
    else {
      debugLine(DEBUG_LEVEL_RETURN_CODES,
                "%x returned on ISRT command using CTDLI()",
                ctdliRC);
      parmCount = 2;
      ctdliRC = CTDLI(&parmCount, PURG, transPCB);
      /* ctdliRC = 0x01dc9901;    */
      if (ctdliRC != 0) {
        debugLine(DEBUG_LEVEL_MINIMUM,
                  "%x returned on PURG command using CTDLI()",
                  ctdliRC);
        values??(0??) = MODULE_NAME;
        values??(1??) = &PURG;
        values??(2??) = &ctdliRC;
        values??(3??) = NULL;
        buildReport("Q102",
                    values);
        sendErrorMessage();
        rc = RETURN_WARN;
      }
      else {
        debugLine(DEBUG_LEVEL_RETURN_CODES,
                  "%x returned on PURG command using CTDLI()",
                  ctdliRC);
      }
    }
  }
  /*******************************************************************/
  /*                                                                 */
  /* We have already notified the master terminal but was have to    */
  /* put message on the DLQ.                                         */
  /*                                                                 */
  /*******************************************************************/
  if (rc > RETURN_OK) {
    putToDLQ(info,
             (MQTM *)&mqtmc2,
             msgBufferLen,
             MQFB_APPL_CANNOT_BE_STARTED);
  }

  return rc;
}
/*********************************************************************/
/* End of function startTrans                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  UT2Headers                                        */
/*                                                                   */
/* Description: Write header line to file CSQQUT2.                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  none                                           */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void UT2Headers() {
  char thedate??(11??);
  char thetime??(9??);
  time_t timenow;
  struct tm *ptr;
  void *values??(4??);

  time(&timenow);
  ptr = localtime(&timenow);
  strftime(thedate, 11, "%Y:%m:%d\n", ptr);
  strftime(thetime,  9, "%H:%M:%S\n", ptr);
  values??(0??) = MODULE_NAME;
  values??(1??) = VERSION;
  values??(2??) = NULL;
  values??(3??) = NULL;
  buildReport("Q150",
              values);
  writeLine(csqqut2);
  values??(0??) = MODULE_NAME;
  values??(1??) = thedate;
  values??(2??) = thetime;
  values??(3??) = NULL;
  buildReport("Q151",
              values);
  writeLine(csqqut2);
  debugLine(DEBUG_LEVEL_FULL, "Written header line to file CSQQUT2");
}
/*********************************************************************/
/* End of function UT2Headers                                        */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name: validateInput                                      */
/*                                                                   */
/* Description: Validates the input parameters, signals error if a   */
/*              required paramter is missing and sets the default    */
/*              when required.                                       */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  params       - ptr to a structure to hold the  */
/*                                   input parameters.               */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           RETURN_OK        - if successful               */
/*                    RETURN_FAIL      - If the program should       */
/*                                       terminate with an error.    */
/*                                                                   */
/*********************************************************************/
short validateInput(INPUT_PARAMS *params) {
  short rc = RETURN_OK;

  if ((params->qmgrName??(0??)  == '\0') ||
      (params->qmgrName??(0??)  == ' ')) {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "WARNING - the queue manager name not given, will " \
              "try using the default");
  }
  if ((params->initQueueName??(0??) == '\0') ||
      (params->initQueueName??(0??) == ' '))  {
    debugLine(DEBUG_LEVEL_MINIMUM,
              "ERROR - the initiation queue name must be given");
    rc = RETURN_FAIL;
  }
  if ((params->lterm??(0??) == '\0') ||
      (params->lterm??(0??) == ' '))  {
    debugLine(DEBUG_LEVEL_MINIMUM,
             "INFORMATION - lterm not given, setting to MASTER");
    memcpy(params->lterm, "MASTER  ", 8);
  }
  if ((strcmp(params->consoleMessagesText, "YES")) ||
      (strcmp(params->consoleMessagesText, "Y"))) {
    params->consoleMessages = 0;
  }
  else {
   debugLine(DEBUG_LEVEL_MINIMUM,
              "Either ConsoleMessage not set or set to a invalid " \
              "value so set");
    debugLine(DEBUG_LEVEL_MINIMUM, "Setting it to NOT.");
    params->consoleMessages = 1;
  }
  if ((params->debugLevel < -2) ||
      (params->debugLevel > DEBUG_LEVEL_FULL)) {
    debugLine(DEBUG_LEVEL_FULL, "Unrecongized debug level of %d so " \
              "turning debug off");
    params->debugLevel = DEBUG_LEVEL_NONE;
  }

  return rc;
}
/*********************************************************************/
/* End of function validateInput                                     */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  writeLine                                         */
/*                                                                   */
/* Description: Writes the report to a file if the file is opened.   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Input Parameters:  csqqut2           - ptr to file to write       */
/*                                        report to.                 */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* InOut Parameters:  none                                           */
/*                                                                   */
/* Returns:           none                                           */
/*                                                                   */
/*********************************************************************/
void writeLine(FILE *csqqut2) {

  if (csqqut2 != NULL) {                /* Write a line              */
    fprintf(csqqut2, "%s\n", report.reportline);
    debugLine(DEBUG_LEVEL_FULL,
              "Writing to CSQQUT2 %s",
              report.reportline);
  }
}
/*********************************************************************/
/* End of function writeLine                                         */
/*********************************************************************/
